'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System.IO
Imports System.Web.Caching
Imports System.Threading
Imports System.Resources
Imports System.Collections.Specialized
Imports System.Diagnostics
Imports System.Reflection
Imports System.Xml
Imports DotNetNuke.Entities.Tabs

Namespace DotNetNuke.Services.Localization

    ''' <summary>
    ''' <para>CultureDropDownTypes allows the user to specify which culture name is displayed in the drop down list that is filled 
    ''' by using one of the helper methods.</para>
    ''' </summary>
    <Serializable()> _
    Public Enum CultureDropDownTypes
        ''' <value>Displays the culture name in the format "&lt;languagefull&gt; (&lt;country/regionfull&gt;) in the .NET Framework language</value>
        DisplayName
        ''' <value>Displays the culture name in the format "&lt;languagefull&gt; (&lt;country/regionfull&gt;) in English</value>
        EnglishName
        ''' <value>Displays the culture identifier</value>
        Lcid
        ''' <value>Displays the culture name in the format "&lt;languagecode2&gt; (&lt;country/regioncode2&gt;)</value>
        Name
        ''' <value>Displays the culture name in the format "&lt;languagefull&gt; (&lt;country/regionfull&gt;) in the language that the culture is set to display</value>
        NativeName
        ''' <value>Displays the IS0 639-1 two letter code</value>
        TwoLetterIsoCode
        ''' <value>Displays the ISO 629-2 three letter code "&lt;languagefull&gt; (&lt;country/regionfull&gt;)</value>
        ThreeLetterIsoCode
    End Enum

    Public Class Localization

#Region "Constants"
        Friend Const keyConst As String = "resourcekey"
        Public Const SystemLocale As String = "en-US"
        Public Const SharedResourceFile As String = ApplicationResourceDirectory + "/SharedResources.resx"
        Public Const LocalResourceDirectory As String = "App_LocalResources"
        Public Const LocalSharedResourceFile As String = "SharedResources.resx"
        Public Const ApplicationResourceDirectory As String = "~/App_GlobalResources"
        Public Const GlobalResourceFile As String = ApplicationResourceDirectory + "/GlobalResources.resx"
        Public Const SupportedLocalesFile As String = ApplicationResourceDirectory + "/Locales.xml"
        Public Const TimezonesFile As String = ApplicationResourceDirectory + "/TimeZones.xml"
        Public Const SystemTimeZoneOffset As Integer = -480
#End Region

#Region "Properties"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CurrentCulture returns the current Culture being used
        ''' is 'key'.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CurrentCulture() As String
            Get
                Return System.Threading.Thread.CurrentThread.CurrentCulture.ToString    ' _CurrentCulture
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The KeyName property returns and caches the name of the key attribute used to lookup resources.
        ''' This can be configured by setting ResourceManagerKey property in the web.config file. The default value for this property
        ''' is 'key'.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Property KeyName() As String
            Get
                Return _defaultKeyName
            End Get
            Set(ByVal Value As String)
                _defaultKeyName = Value
                If _defaultKeyName Is Nothing Or _defaultKeyName = String.Empty Then
                    _defaultKeyName = keyConst
                End If
            End Set
        End Property

#End Region

#Region "Private Members"
        Private Shared _defaultKeyName As String = keyConst
        Private Shared _timeZoneListItems() As ListItem
#End Region

#Region "Private Methods"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetResource is used by GetString to load the resources Hashtable
        ''' </summary>
        ''' <remarks>
        '''     The priority of cultures is User > Portal Default > FallBack
        '''
        '''     Therefore the logic used is the following:
        '''     1. Load the fallback language resources first (these are known
        '''         to exist, thus making sure that all keys are loaded)
        '''     2. Next load the Portal's default language if different (any keys that
        '''         are present overwrite the fallback keys).
        '''     3. Last load the user's language if different from either of the above
        '''         (again any keys that are present overwrite the fallback/default keys).
        '''
        ''' </remarks>
        ''' <param name="ResourceFileRoot">The Local Resource root</param>
        ''' <param name="objPortalSettings">The current portals Portal Settings</param>
        ''' <returns>The rsources Hashtable </returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Shared Function GetResource(ByVal ResourceFileRoot As String, ByVal objPortalSettings As PortalSettings) As Hashtable
            Return GetResource(ResourceFileRoot, objPortalSettings, Nothing)
        End Function
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetResource is used by GetString to load the resources Hashtable
        ''' </summary>
        ''' <remarks>
        '''     The priority of cultures is User > Portal Default > FallBack
        '''
        '''     Therefore the logic used is the following:
        '''     1. Load the fallback language resources first (these are known
        '''         to exist, thus making sure that all keys are loaded)
        '''     2. Next load the Portal's default language if different (any keys that
        '''         are present overwrite the fallback keys).
        '''     3. Last load the user's language if different from either of the above
        '''         (again any keys that are present overwrite the fallback/default keys).
        '''
        ''' </remarks>
        ''' <param name="ResourceFileRoot">The Local Resource root</param>
        ''' <param name="objPortalSettings">The current portals Portal Settings</param>
        ''' <param name="Language">A specific language used to get the resource</param>
        ''' <returns>The resources Hashtable </returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Shared Function GetResource(ByVal ResourceFileRoot As String, ByVal objPortalSettings As PortalSettings, ByVal strLanguage As String) As Hashtable

            Dim resources As Hashtable
            Dim defaultLanguage As String = objPortalSettings.DefaultLanguage.ToLower
            Dim userLanguage As String
            If strLanguage Is Nothing Then
                userLanguage = Thread.CurrentThread.CurrentCulture.ToString.ToLower
            Else
                userLanguage = strLanguage
            End If
            Dim fallbackLanguage As String
            Dim userFile As String
            Dim defaultFile As String
            Dim fallbackFile As String

            ' Ensure the user has a language set
            If userLanguage = "" Then
                userLanguage = defaultLanguage
            End If

            If Not GetSupportedLocales(userLanguage) Is Nothing Then
                fallbackLanguage = GetSupportedLocales(userLanguage).Fallback.ToLower
            Else
                fallbackLanguage = SystemLocale
            End If

            'Get the filename for the userlanguage version of the resource file
            userFile = GetResourceFileName(ResourceFileRoot, userLanguage)

            'Set the cachekey as the userFile + the PortalId
            Dim cacheKey As String = userFile.ToLower + objPortalSettings.PortalId.ToString

            'Attempt to get the resources from the cache
            If DataCache.GetCache(cacheKey) Is Nothing Then
                ' resources not in Cache so load from Files

                'Create resources Hashtable
                resources = New Hashtable

                'First Load the Fallback Language ensuring that the keys are loaded
                fallbackFile = GetResourceFileName(ResourceFileRoot, fallbackLanguage)
                resources = LoadResource(resources, fallbackLanguage, cacheKey, fallbackFile, False, objPortalSettings)
                ' Add any portal level customizations
                resources = LoadResource(resources, fallbackLanguage, cacheKey, fallbackFile, True, objPortalSettings)

                'if the defaultLanguage is different, load it
                If defaultLanguage <> "" AndAlso defaultLanguage <> fallbackLanguage Then
                    defaultFile = GetResourceFileName(ResourceFileRoot, defaultLanguage)
                    resources = LoadResource(resources, defaultLanguage, cacheKey, defaultFile, False, objPortalSettings)
                    ' Add any portal level customizations
                    resources = LoadResource(resources, defaultLanguage, cacheKey, defaultFile, True, objPortalSettings)
                End If

                ' If the user language is different load it
                If userLanguage <> defaultLanguage AndAlso userLanguage <> fallbackLanguage Then
                    resources = LoadResource(resources, userLanguage, cacheKey, userFile, False, objPortalSettings)
                    ' Add any portal level customizations
                    resources = LoadResource(resources, userLanguage, cacheKey, userFile, True, objPortalSettings)
                End If
                If Common.Globals.PerformanceSetting = Common.Globals.PerformanceSettings.NoCaching Then
                    Return resources
                End If
            End If

            Return CType(DataCache.GetCache(cacheKey), Hashtable)
        End Function    'GetResource

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetResourceFileName is used to build the resource file name according to the
        ''' language
        ''' </summary>
        ''' <param name="language">The language</param>
        ''' <param name="ResourceFileRoot">The resource file root</param>
        ''' <returns>The language specific resource file</returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Shared Function GetResourceFileName(ByVal ResourceFileRoot As String, ByVal language As String) As String

            Dim ResourceFile As String

            If Not ResourceFileRoot Is Nothing Then
                If language = Services.Localization.Localization.SystemLocale.ToLower Or language = "" Then
                    Select Case Right(ResourceFileRoot, 5).ToLower
                        Case ".resx"
                            ResourceFile = ResourceFileRoot
                        Case ".ascx"
                            ResourceFile = ResourceFileRoot & ".resx"
                        Case Else
                            ResourceFile = ResourceFileRoot + ".ascx.resx"       ' a portal module
                    End Select
                Else
                    Select Case Right(ResourceFileRoot, 5).ToLower
                        Case ".resx"
                            ResourceFile = ResourceFileRoot.Replace(".resx", "." + language + ".resx")
                        Case ".ascx"
                            ResourceFile = ResourceFileRoot.Replace(".ascx", ".ascx." + language + ".resx")
                        Case Else
                            ResourceFile = ResourceFileRoot + ".ascx." + language + ".resx"       ' a portal module
                    End Select
                End If
            Else
                If language = Services.Localization.Localization.SystemLocale.ToLower Then
                    ResourceFile = Services.Localization.Localization.SharedResourceFile
                Else
                    ResourceFile = Services.Localization.Localization.SharedResourceFile.Replace(".resx", "." + language + ".resx")
                End If
            End If

            Return ResourceFile

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LoadResource is used by getResource to add resources to the resources Hashtable
        ''' </summary>
        ''' <param name="target">The Hashtable to add resources to</param>
        ''' <param name="language">The language to load</param>
        ''' <param name="cacheKey">The cache key</param>
        ''' <param name="ResourceFile">The language specific resource file</param>
        ''' <param name="CheckCustomCulture">A flag to determine if portal specific resources are to be checked</param>
        ''' <param name="objPortalSettings">The current portals Portal Settings</param>
        ''' <returns>The updated resources Hashtable </returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Shared Function LoadResource(ByVal target As Hashtable, ByVal language As String, ByVal cacheKey As String, ByVal ResourceFile As String, ByVal CheckCustomCulture As Boolean, ByVal objPortalSettings As PortalSettings) As Hashtable

            Dim filePath As String
            Dim f As String

            'Are we looking for customised resources
            If CheckCustomCulture Then
                f = ResourceFile.Replace(".resx", ".Portal-" + objPortalSettings.PortalId.ToString + ".resx")
            Else
                f = ResourceFile
            End If

            'If the filename is empty or the file does not exist return the Hashtable
            If f Is Nothing OrElse File.Exists(HttpContext.Current.Server.MapPath(f)) = False Then
                Return target
            End If

            filePath = HttpContext.Current.Server.MapPath(f)

            Dim dp As New CacheDependency(filePath)
            Dim xmlLoaded As Boolean = False
            Dim d As New XmlDocument
            Try
                d.Load(filePath)
                xmlLoaded = True
            Catch    'exc As Exception
                xmlLoaded = False
            End Try
            If xmlLoaded Then
                Dim n As XmlNode
                For Each n In d.SelectNodes("root/data")
                    If n.NodeType <> XmlNodeType.Comment Then
                        Dim val As String = n.SelectSingleNode("value").InnerText
                        If target(n.Attributes("name").Value) Is Nothing Then
                            target.Add(n.Attributes("name").Value, val)
                        Else
                            target(n.Attributes("name").Value) = val
                        End If

                    End If
                Next n

                Dim CacheMins As Integer
                CacheMins = 3 * Convert.ToInt32(Common.Globals.PerformanceSetting)

                If CacheMins > 0 Then
                    DataCache.SetCache(cacheKey, target, dp, Date.MaxValue, New TimeSpan(0, CacheMins, 0))
                End If
            End If
            Return target
        End Function    'LoadResource 

        Private Shared Function LocalizeTabStripDetails(ByVal arrTabStripDetails As ArrayList) As ArrayList
            Dim i As Integer
            For i = 0 To arrTabStripDetails.Count - 1
                Dim objTab As TabInfo = CType(arrTabStripDetails(i), TabInfo)
                If objTab.IsAdminTab Then
                    Dim strLocalizedTabName As String = Services.Localization.Localization.GetString(objTab.TabName + ".String", Services.Localization.Localization.GlobalResourceFile)
                    If strLocalizedTabName <> "" Then
                        objTab.TabName = strLocalizedTabName
                        objTab.Title = ""
                    End If
                End If
            Next
            Return arrTabStripDetails
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the TimeZone file name for a given resource and language
        ''' </summary>
        ''' <param name="filename">Resource File</param>
        ''' <param name="language">Language</param>
        ''' <returns>Localized File Name</returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[vmasanas]	04/10/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function TimeZoneFile(ByVal filename As String, ByVal language As String) As String
            If language = Services.Localization.Localization.SystemLocale Then
                Return filename
            Else
                Return filename.Substring(0, filename.Length - 4) + "." + language + ".xml"
            End If
        End Function

#End Region

#Region "Public Methods"

        Public Function AddLocale(ByVal Key As String, ByVal Name As String) As String
            Dim node As XmlNode
            Dim resDoc As New XmlDocument
            Dim a As XmlAttribute
            Dim result As String

            resDoc.Load(HttpContext.Current.Server.MapPath(Localization.SupportedLocalesFile))
            node = resDoc.SelectSingleNode("//root/language[@key='" + Key + "']")
            If Not node Is Nothing Then
                result = "Duplicate.ErrorMessage"
            Else
                node = resDoc.CreateElement("language")
                a = resDoc.CreateAttribute("name")
                a.Value = Name
                node.Attributes.Append(a)
                a = resDoc.CreateAttribute("key")
                a.Value = Key
                node.Attributes.Append(a)
                a = resDoc.CreateAttribute("fallback")
                a.Value = Services.Localization.Localization.SystemLocale
                node.Attributes.Append(a)
                resDoc.SelectSingleNode("//root").AppendChild(node)

                Try
                    resDoc.Save(HttpContext.Current.Server.MapPath(Localization.SupportedLocalesFile))
                    If Not File.Exists(HttpContext.Current.Server.MapPath(TimeZoneFile(Localization.TimezonesFile, Key))) Then
                        File.Copy(HttpContext.Current.Server.MapPath(Localization.TimezonesFile), HttpContext.Current.Server.MapPath(TimeZoneFile(Localization.TimezonesFile, Key)))
                        result = "NewLocale.ErrorMessage"
                    End If
                    result = ""
                Catch
                    result = "Save.ErrorMessage"
                End Try
            End If
            Return result

        End Function

        Public Function GetFixedCurrency(ByVal Expression As Decimal, ByVal Culture As String, Optional ByVal NumDigitsAfterDecimal As Integer = -1, Optional ByVal IncludeLeadingDigit As Microsoft.VisualBasic.TriState = TriState.UseDefault, Optional ByVal UseParensForNegativeNumbers As Microsoft.VisualBasic.TriState = TriState.UseDefault, Optional ByVal GroupDigits As Microsoft.VisualBasic.TriState = TriState.UseDefault) As String
            Dim oldCurrentCulture As String = CurrentCulture
            Dim newCulture As System.Globalization.CultureInfo = New System.Globalization.CultureInfo(Culture)
            System.Threading.Thread.CurrentThread.CurrentUICulture = newCulture
            System.Threading.Thread.CurrentThread.CurrentCulture = newCulture
            Dim currencyStr As String = FormatCurrency(Expression, NumDigitsAfterDecimal, IncludeLeadingDigit, UseParensForNegativeNumbers, GroupDigits)
            Return currencyStr
            Dim oldCulture As System.Globalization.CultureInfo = New System.Globalization.CultureInfo(oldCurrentCulture)
            System.Threading.Thread.CurrentThread.CurrentUICulture = oldCulture
            System.Threading.Thread.CurrentThread.CurrentCulture = oldCulture
        End Function

        Public Function GetFixedDate(ByVal Expression As Date, ByVal Culture As String, Optional ByVal NamedFormat As Microsoft.VisualBasic.DateFormat = DateFormat.GeneralDate, Optional ByVal IncludeLeadingDigit As Microsoft.VisualBasic.TriState = TriState.UseDefault, Optional ByVal UseParensForNegativeNumbers As Microsoft.VisualBasic.TriState = TriState.UseDefault, Optional ByVal GroupDigits As Microsoft.VisualBasic.TriState = TriState.UseDefault) As String
            Dim oldCurrentCulture As String = CurrentCulture
            Dim newCulture As System.Globalization.CultureInfo = New System.Globalization.CultureInfo(Culture)
            System.Threading.Thread.CurrentThread.CurrentUICulture = newCulture
            System.Threading.Thread.CurrentThread.CurrentCulture = newCulture
            Dim dateStr As String = FormatDateTime(Expression, NamedFormat)
            Return dateStr
            Dim oldCulture As System.Globalization.CultureInfo = New System.Globalization.CultureInfo(oldCurrentCulture)
            System.Threading.Thread.CurrentThread.CurrentUICulture = oldCulture
            System.Threading.Thread.CurrentThread.CurrentCulture = oldCulture
        End Function

        Public Shared Function GetResourceFile(ByVal Ctrl As Control, ByVal FileName As String) As String
            Return Ctrl.TemplateSourceDirectory + "/" + Services.Localization.Localization.LocalResourceDirectory + "/" + FileName
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <overloads>One of five overloads</overloads>
        ''' <summary>
        ''' GetString gets the localized string corresponding to the resourcekey
        ''' </summary>
        ''' <param name="name">The resourcekey to find</param>
        ''' <returns>The localized Text</returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetString(ByVal name As String) As String

            Dim objPortalSettings As PortalSettings = CType(HttpContext.Current.Items("PortalSettings"), PortalSettings)

            Return GetString(name, Nothing, objPortalSettings, Nothing)

        End Function    'GetString

        ''' -----------------------------------------------------------------------------
        ''' <overloads>One of five overloads</overloads>
        ''' <summary>
        ''' GetString gets the localized string corresponding to the resourcekey
        ''' </summary>
        ''' <param name="name">The resourcekey to find</param>
        ''' <param name="objPortalSettings">The current portals Portal Settings</param>
        ''' <returns>The localized Text</returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetString(ByVal name As String, ByVal objPortalSettings As PortalSettings) As String

            Return GetString(name, Nothing, objPortalSettings, Nothing)

        End Function    'GetString

        ''' -----------------------------------------------------------------------------
        ''' <overloads>One of five overloads</overloads>
        ''' <summary>
        ''' GetString gets the localized string corresponding to the resourcekey
        ''' </summary>
        ''' <param name="name">The resourcekey to find</param>
        ''' <param name="ResourceFileRoot">The Local Resource root</param>
        ''' <returns>The localized Text</returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetString(ByVal name As String, ByVal ResourceFileRoot As String) As String

            Dim objPortalSettings As PortalSettings = CType(HttpContext.Current.Items("PortalSettings"), PortalSettings)

            Return GetString(name, ResourceFileRoot, objPortalSettings, Nothing)

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <overloads>One of five overloads</overloads>
        ''' <summary>
        ''' GetString gets the localized string corresponding to the resourcekey
        ''' </summary>
        ''' <param name="name">The resourcekey to find</param>
        ''' <param name="ResourceFileRoot">The Local Resource root</param>
        ''' <param name="strLanguage">A specific language to lookup the string</param>
        ''' <returns>The localized Text</returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetString(ByVal name As String, ByVal ResourceFileRoot As String, ByVal strlanguage As String) As String

            Dim objPortalSettings As PortalSettings = CType(HttpContext.Current.Items("PortalSettings"), PortalSettings)

            Return GetString(name, ResourceFileRoot, objPortalSettings, strlanguage)

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <overloads>One of five overloads</overloads>
        ''' <summary>
        ''' GetString gets the localized string corresponding to the resourcekey
        ''' </summary>
        ''' <param name="name">The resourcekey to find</param>
        ''' <param name="ResourceFileRoot">The Local Resource root</param>
        ''' <param name="objPortalSettings">The current portals Portal Settings</param>
        ''' <param name="strLanguage">A specific language to lookup the string</param>
        ''' <returns>The localized Text</returns>
        ''' <history>
        ''' 	[cnurse]	10/06/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetString(ByVal name As String, ByVal ResourceFileRoot As String, ByVal objPortalSettings As PortalSettings, ByVal strLanguage As String) As String

            'Load the Local Resource Files resources
            Dim resources As Hashtable = GetResource(ResourceFileRoot, objPortalSettings, strLanguage)

            'make the default translation property ".Text"
            If name.IndexOf(".") < 1 Then
                name += ".Text"
            End If

            'If the key can't be found try the Shared Resource Files resources
            If resources Is Nothing OrElse resources(name) Is Nothing Then
                resources = GetResource(Localization.SharedResourceFile, objPortalSettings)
            End If

            'If the key still can't be found then it doesn't exist in the Localization Resources
            If resources(name) Is Nothing Then
                'This has to be replaced with a function that will add the key to the xml file (autofill)
                'Throw New Exception("Value not found in resources.xml for: " + name)
                Debug.WriteLine("Value not found in resources.xml for: " + name + " in " + ResourceFileRoot)
                'simply return the localization key if nothing is found in the resource file.
                'This will at least prevent an error 
                'check for setting in web.config
                Dim showMissingKeys As Boolean
                If System.Configuration.ConfigurationSettings.AppSettings("ShowMissingKeys") Is Nothing Then
                    showMissingKeys = False
                Else
                    showMissingKeys = Boolean.Parse(System.Configuration.ConfigurationSettings.AppSettings("ShowMissingKeys"))
                End If
                If showMissingKeys Then
                    Return "RESX:" & name
                End If
            End If

            Return CStr(resources(name))

        End Function    'GetString

        ''' <summary>
        ''' <para>GetSupportedLocales returns the list of locales from the locales.xml file.</para>
        ''' </summary>
        Public Shared Function GetSupportedLocales() As LocaleCollection

            Dim cacheKey As String = "dotnetnuke-supportedlocales"

            If DataCache.GetCache(cacheKey) Is Nothing Then
                Dim supportedLocales As New LocaleCollection
                Dim filePath As String = Common.Globals.ApplicationMapPath & Localization.SupportedLocalesFile.Substring(1).Replace("/", "\")

                If Not File.Exists(filePath) Then
                    Try
                        ' First access to locales.xml, create using template
                        File.Copy(Common.Globals.ApplicationMapPath & ApplicationResourceDirectory.Substring(1).Replace("/", "\") & "\Locales.xml.resources", Common.Globals.ApplicationMapPath & Localization.SupportedLocalesFile.Substring(1).Replace("/", "\"))
                    Catch
                        Dim objLocale As New Locale
                        objLocale.Text = "English"
                        objLocale.Code = "en-US"
                        objLocale.Fallback = ""
                        supportedLocales.Add("en-US", objLocale)
                        Return supportedLocales 'Will be Empty and not cached
                    End Try
                End If

                Dim dp As New CacheDependency(filePath)

                Dim d As New XmlDocument
                d.Load(filePath)

                Dim n As XmlNode
                For Each n In d.SelectNodes("root/language")
                    If n.NodeType <> XmlNodeType.Comment Then
                        Dim objLocale As New Locale
                        objLocale.Text = n.Attributes("name").Value
                        objLocale.Code = n.Attributes("key").Value
                        objLocale.Fallback = n.Attributes("fallback").Value

                        supportedLocales.Add(n.Attributes("key").Value, objLocale)
                    End If
                Next n
                If Common.Globals.PerformanceSetting <> Common.Globals.PerformanceSettings.NoCaching Then
                    DataCache.SetCache(cacheKey, supportedLocales, dp)
                Else
                    Return supportedLocales
                End If

            End If

            Return CType(DataCache.GetCache(cacheKey), LocaleCollection)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a SystemMessage.
        ''' </summary>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Supported tags:
        ''' - All fields from HostSettings table in the form of: [Host:<b>field</b>]
        ''' - All properties defined in <see cref="T:DotNetNuke.PortalInfo" /> in the form of: [Portal:<b>property</b>]
        ''' - [Portal:URL]: The base URL for the portal
        ''' - All properties defined in <see cref="T:DotNetNuke.UserInfo" /> in the form of: [User:<b>property</b>]
        ''' - All values stored in the user profile in the form of: [Profile:<b>key</b>]
        ''' - [User:VerificationCode]: User verification code for verified registrations
        ''' - [Date:Current]: Current date
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal objPortal As PortalSettings, ByVal MessageName As String) As String
            Return GetSystemMessage(Nothing, objPortal, MessageName, Nothing, GlobalResourceFile, Nothing)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a SystemMessage.
        ''' </summary>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <param name="objUser">Reference to the user used to personalize the message.</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Supported tags:
        ''' - All fields from HostSettings table in the form of: [Host:<b>field</b>]
        ''' - All properties defined in <see cref="T:DotNetNuke.PortalInfo" /> in the form of: [Portal:<b>property</b>]
        ''' - [Portal:URL]: The base URL for the portal
        ''' - All properties defined in <see cref="T:DotNetNuke.UserInfo" /> in the form of: [User:<b>property</b>]
        ''' - All values stored in the user profile in the form of: [Profile:<b>key</b>]
        ''' - [User:VerificationCode]: User verification code for verified registrations
        ''' - [Date:Current]: Current date
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal objPortal As PortalSettings, ByVal MessageName As String, ByVal objUser As UserInfo) As String
            Return GetSystemMessage(Nothing, objPortal, MessageName, objUser, GlobalResourceFile, Nothing)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        '''         ''' Gets a SystemMessage.
        ''' </summary>
        ''' <param name="strLanguage">A specific language to get the SystemMessage for.</param>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <param name="objUser">Reference to the user used to personalize the message.</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Supported tags:
        ''' - All fields from HostSettings table in the form of: [Host:<b>field</b>]
        ''' - All properties defined in <see cref="T:DotNetNuke.PortalInfo" /> in the form of: [Portal:<b>property</b>]
        ''' - [Portal:URL]: The base URL for the portal
        ''' - All properties defined in <see cref="T:DotNetNuke.UserInfo" /> in the form of: [User:<b>property</b>]
        ''' - All values stored in the user profile in the form of: [Profile:<b>key</b>]
        ''' - [User:VerificationCode]: User verification code for verified registrations
        ''' - [Date:Current]: Current date
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal strLanguage As String, ByVal objPortal As PortalSettings, ByVal MessageName As String, ByVal objUser As UserInfo) As String
            Return GetSystemMessage(strLanguage, objPortal, MessageName, objUser, GlobalResourceFile, Nothing)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a SystemMessage.
        ''' </summary>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <param name="ResourceFile">The root name of the Resource File where the localized
        '''   text can be found</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Supported tags:
        ''' - All fields from HostSettings table in the form of: [Host:<b>field</b>]
        ''' - All properties defined in <see cref="T:DotNetNuke.PortalInfo" /> in the form of: [Portal:<b>property</b>]
        ''' - [Portal:URL]: The base URL for the portal
        ''' - All properties defined in <see cref="T:DotNetNuke.UserInfo" /> in the form of: [User:<b>property</b>]
        ''' - All values stored in the user profile in the form of: [Profile:<b>key</b>]
        ''' - [User:VerificationCode]: User verification code for verified registrations
        ''' - [Date:Current]: Current date
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal objPortal As PortalSettings, ByVal MessageName As String, ByVal ResourceFile As String) As String
            Return GetSystemMessage(Nothing, objPortal, MessageName, Nothing, ResourceFile, Nothing)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a SystemMessage.
        ''' </summary>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <param name="objUser">Reference to the user used to personalize the message.</param>
        ''' <param name="ResourceFile">The root name of the Resource File where the localized
        '''   text can be found</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Supported tags:
        ''' - All fields from HostSettings table in the form of: [Host:<b>field</b>]
        ''' - All properties defined in <see cref="T:DotNetNuke.PortalInfo" /> in the form of: [Portal:<b>property</b>]
        ''' - [Portal:URL]: The base URL for the portal
        ''' - All properties defined in <see cref="T:DotNetNuke.UserInfo" /> in the form of: [User:<b>property</b>]
        ''' - All values stored in the user profile in the form of: [Profile:<b>key</b>]
        ''' - [User:VerificationCode]: User verification code for verified registrations
        ''' - [Date:Current]: Current date
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal objPortal As PortalSettings, ByVal MessageName As String, ByVal objUser As UserInfo, ByVal ResourceFile As String) As String
            Return GetSystemMessage(Nothing, objPortal, MessageName, objUser, ResourceFile, Nothing)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a SystemMessage passing extra custom parameters to personalize.
        ''' </summary>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <param name="ResourceFile">The root name of the Resource File where the localized
        '''   text can be found</param>
        ''' <param name="Custom">An ArrayList with replacements for custom tags.</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Custom tags are of the form <b>[Custom:n]</b>, where <b>n</b> is the zero based index which 
        ''' will be used to find the replacement value in <b>Custom</p> parameter.
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        '''     [cnurse]    10/06/2004  Moved from SystemMessages to Localization
        '''     [DanCaron]  10/27/2004  Simplified Profile replacement, added Membership replacement
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal objPortal As PortalSettings, ByVal MessageName As String, ByVal ResourceFile As String, ByVal Custom As ArrayList) As String
            Return GetSystemMessage(Nothing, objPortal, MessageName, Nothing, ResourceFile, Custom)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a SystemMessage passing extra custom parameters to personalize.
        ''' </summary>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <param name="objUser">Reference to the user used to personalize the message.</param>
        ''' <param name="ResourceFile">The root name of the Resource File where the localized
        '''   text can be found</param>
        ''' <param name="Custom">An ArrayList with replacements for custom tags.</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Custom tags are of the form <b>[Custom:n]</b>, where <b>n</b> is the zero based index which 
        ''' will be used to find the replacement value in <b>Custom</p> parameter.
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        '''     [cnurse]    10/06/2004  Moved from SystemMessages to Localization
        '''     [DanCaron]  10/27/2004  Simplified Profile replacement, added Membership replacement
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal objPortal As PortalSettings, ByVal MessageName As String, ByVal objUser As UserInfo, ByVal ResourceFile As String, ByVal Custom As ArrayList) As String
            Return GetSystemMessage(Nothing, objPortal, MessageName, objUser, ResourceFile, Custom)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a SystemMessage passing extra custom parameters to personalize.
        ''' </summary>
        ''' <param name="strLanguage">A specific language to get the SystemMessage for.</param>
        ''' <param name="objPortal">The portal settings for the portal to which the message will affect.</param>
        ''' <param name="MessageName">The message tag which identifies the SystemMessage.</param>
        ''' <param name="objUser">Reference to the user used to personalize the message.</param>
        ''' <param name="ResourceFile">The root name of the Resource File where the localized
        '''   text can be found</param>
        ''' <param name="Custom">An ArrayList with replacements for custom tags.</param>
        ''' <returns>The message body with all tags replaced.</returns>
        ''' <remarks>
        ''' Custom tags are of the form <b>[Custom:n]</b>, where <b>n</b> is the zero based index which 
        ''' will be used to find the replacement value in <b>Custom</p> parameter.
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        '''     [cnurse]    10/06/2004  Moved from SystemMessages to Localization
        '''     [DanCaron]  10/27/2004  Simplified Profile replacement, added Membership replacement
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetSystemMessage(ByVal strLanguage As String, ByVal objPortal As PortalSettings, ByVal MessageName As String, ByVal objUser As UserInfo, ByVal ResourceFile As String, ByVal Custom As ArrayList) As String

            Dim strMessageValue As String
            strMessageValue = GetString(MessageName, ResourceFile, strLanguage)

            If strMessageValue <> "" Then

                Dim strKey As String

                ' host values
                If InStr(1, strMessageValue, "Host:", CompareMethod.Text) <> 0 Then
                    Dim objHostSettings As Hashtable = Entities.Host.HostSettings.GetSecureHostSettings
                    For Each strKey In objHostSettings.Keys
                        If InStr(1, strMessageValue, "[Host:" & strKey & "]", CompareMethod.Text) <> 0 Then
                            strMessageValue = Replace(strMessageValue, "[Host:" & strKey & "]", objHostSettings(strKey).ToString, , , CompareMethod.Text)
                        End If
                    Next
                End If

                ' get portal values
                If InStr(1, strMessageValue, "Portal:", CompareMethod.Text) <> 0 Then
                    If Not objPortal Is Nothing Then
                        strMessageValue = PersonalizeSystemMessage(strMessageValue, "Portal:", objPortal, GetType(PortalSettings))
                    End If
                    strMessageValue = Replace(strMessageValue, "[Portal:URL]", objPortal.PortalAlias.HTTPAlias, , , CompareMethod.Text)
                End If

                ' get user values
                If (Not objUser Is Nothing) And (Not objPortal Is Nothing) Then
                    If InStr(1, strMessageValue, "User:", CompareMethod.Text) <> 0 Then
                        strMessageValue = PersonalizeSystemMessage(strMessageValue, "User:", objUser, GetType(UserInfo))
                        strMessageValue = Replace(strMessageValue, "[User:VerificationCode]", objPortal.PortalId.ToString & "-" & objUser.UserID.ToString, , , CompareMethod.Text)
                    End If
                    If InStr(1, strMessageValue, "Membership:", CompareMethod.Text) <> 0 Then
                        strMessageValue = PersonalizeSystemMessage(strMessageValue, "Membership:", objUser.Membership, GetType(UserMembership))
                    End If
                    If InStr(1, strMessageValue, "Profile:", CompareMethod.Text) <> 0 Then
                        strMessageValue = PersonalizeSystemMessage(strMessageValue, "Profile:", objUser.Profile, GetType(UserProfile))
                    End If
                End If

                ' custom
                If InStr(1, strMessageValue, "Custom:", CompareMethod.Text) <> 0 Then
                    If Not Custom Is Nothing Then
                        Dim intIndex As Integer
                        For intIndex = 0 To Custom.Count - 1
                            strMessageValue = Replace(strMessageValue, "[Custom:" & intIndex.ToString & "]", Custom(intIndex).ToString, , , CompareMethod.Text)
                        Next intIndex
                    End If
                End If

                ' constants
                Dim ci As CultureInfo
                If Not strLanguage Is Nothing Then
                    ci = New CultureInfo(strLanguage)
                Else
                    ci = New CultureInfo(Thread.CurrentThread.CurrentCulture.ToString.ToLower)
                End If

                strMessageValue = Replace(strMessageValue, "[Date:Current]", Now().ToString("D", ci), , , CompareMethod.Text)

            End If

            Return strMessageValue

        End Function

        '' -----------------------------------------------------------------------------
        '' <summary>
        '' GetTimeZones gets a collection of Tme Zones in the relevant language
        '' </summary>
        '' <param name="language">Language</param>
        ''	<returns>The TimeZones as a Name/Value Collection</returns>
        '' <history>
        '' 	[cnurse]	10/29/2004	Modified to exit gracefully if no relevant file
        '' </history>
        '' -----------------------------------------------------------------------------
        Public Shared Function GetTimeZones(ByVal language As String) As NameValueCollection
            language = language.ToLower
            Dim timeZones As NameValueCollection
            Dim cacheKey As String = "dotnetnuke-" + language + "-timezones"

            Dim TranslationFile As String

            If language = Services.Localization.Localization.SystemLocale.ToLower Then
                TranslationFile = Services.Localization.Localization.TimezonesFile
            Else
                TranslationFile = Services.Localization.Localization.TimezonesFile.Replace(".xml", "." + language + ".xml")
            End If

            If DataCache.GetCache(cacheKey) Is Nothing Then
                Dim filePath As String = HttpContext.Current.Server.MapPath(TranslationFile)
                timeZones = New NameValueCollection
                If File.Exists(filePath) = False Then
                    Return timeZones
                End If
                Dim dp As New CacheDependency(filePath)
                Try
                    Dim d As New XmlDocument
                    d.Load(filePath)

                    Dim n As XmlNode
                    For Each n In d.SelectSingleNode("root").ChildNodes
                        If n.NodeType <> XmlNodeType.Comment Then
                            timeZones.Add(n.Attributes("name").Value, n.Attributes("key").Value)
                        End If
                    Next n
                Catch ex As Exception

                End Try
                If Common.Globals.PerformanceSetting = Common.Globals.PerformanceSettings.NoCaching Then
                    Return timeZones
                Else
                    DataCache.SetCache(cacheKey, timeZones, dp)
                End If
            End If

            Return CType(DataCache.GetCache(cacheKey), NameValueCollection)
        End Function    'GetTimeZones

        ''' <summary>
        ''' <para>LoadCultureDropDownList loads a DropDownList with the list of supported cultures
        ''' based on the languages defined in the supported locales file</para>
        ''' </summary>
        ''' <param name="list">DropDownList to load</param>
        ''' <param name="displayType">Format of the culture to display. Must be one the CultureDropDownTypes values. 
        ''' <see cref="CultureDropDownTypes"/> for list of allowable values</param>
        ''' <param name="selectedValue">Name of the default culture to select</param>
        Public Shared Sub LoadCultureDropDownList(ByVal list As DropDownList, ByVal displayType As CultureDropDownTypes, ByVal selectedValue As String)

            Dim supportedLanguages As LocaleCollection = GetSupportedLocales()
            Dim _cultureListItems() As ListItem = New ListItem(supportedLanguages.Count - 1) {}
            Dim _cultureListItemsType As CultureDropDownTypes = displayType
            Dim objPortalSettings As PortalSettings = PortalController.GetCurrentPortalSettings()
            Dim xmlLocales As New XmlDocument
            Dim bXmlLoaded As Boolean = False
            Dim intAdded As Integer = 0

            If File.Exists(HttpContext.Current.Server.MapPath(ApplicationResourceDirectory + "/Locales.Portal-" + objPortalSettings.PortalId.ToString + ".xml")) Then
                Try
                    xmlLocales.Load(HttpContext.Current.Server.MapPath(ApplicationResourceDirectory + "/Locales.Portal-" + objPortalSettings.PortalId.ToString + ".xml"))
                    bXmlLoaded = True
                Catch
                End Try
            End If

            Dim i As Integer
            For i = 0 To supportedLanguages.Count - 1

                ' Only load active locales
                If Not bXmlLoaded OrElse xmlLocales.SelectSingleNode("//locales/inactive/locale[.='" + CType(supportedLanguages(i).Value, Locale).Code + "']") Is Nothing Then
                    ' Create a CultureInfo class based on culture
                    Dim info As CultureInfo = CultureInfo.CreateSpecificCulture(CType(supportedLanguages(i).Value, Locale).Code)

                    ' Create and initialize a new ListItem
                    Dim item As New ListItem
                    item.Value = CType(supportedLanguages(i).Value, Locale).Code

                    ' Based on the display type desired by the user, select the correct property
                    Select Case displayType
                        Case CultureDropDownTypes.EnglishName
                            item.Text = info.EnglishName
                        Case CultureDropDownTypes.Lcid
                            item.Text = info.LCID.ToString()
                        Case CultureDropDownTypes.Name
                            item.Text = info.Name
                        Case CultureDropDownTypes.NativeName
                            item.Text = info.NativeName
                        Case CultureDropDownTypes.TwoLetterIsoCode
                            item.Text = info.TwoLetterISOLanguageName
                        Case CultureDropDownTypes.ThreeLetterIsoCode
                            item.Text = info.ThreeLetterISOLanguageName
                        Case Else
                            item.Text = info.DisplayName
                    End Select
                    _cultureListItems(intAdded) = item
                    intAdded += 1
                End If
            Next i

            ' If the drop down list already has items, clear the list
            If list.Items.Count > 0 Then
                list.Items.Clear()
            End If

            ReDim Preserve _cultureListItems(intAdded - 1)
            ' add the items to the list
            list.Items.AddRange(_cultureListItems)

            ' select the default item
            If Not selectedValue Is Nothing Then
                Dim item As ListItem = list.Items.FindByValue(selectedValue)
                If Not item Is Nothing Then
                    list.SelectedIndex = -1
                    item.Selected = True
                End If
            End If
        End Sub    'LoadCultureDropDownList

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LoadTimeZoneDropDownList loads a drop down list with the Timezones
        ''' </summary>
        ''' <param name="list">The list to load</param>
        ''' <param name="language">Language</param>
        ''' <param name="selectedValue">The selected Time Zone</param>
        ''' <history>
        ''' 	[cnurse]	10/29/2004	documented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub LoadTimeZoneDropDownList(ByVal list As DropDownList, ByVal language As String, ByVal selectedValue As String)

            Dim timeZones As NameValueCollection = GetTimeZones(language)
            'If no Timezones defined get the System Locale Time Zones
            If timeZones.Count = 0 Then
                timeZones = GetTimeZones(Services.Localization.Localization.SystemLocale.ToLower)
            End If
            Dim i As Integer
            For i = 0 To timeZones.Keys.Count - 1
                list.Items.Add(New ListItem(timeZones.GetKey(i).ToString(), timeZones.Get(i).ToString()))
            Next i

            ' select the default item
            If Not selectedValue Is Nothing Then
                Dim item As ListItem = list.Items.FindByValue(selectedValue)
                If item Is Nothing Then
                    'Try system default
                    item = list.Items.FindByValue(SystemTimeZoneOffset.ToString)
                End If
                If Not item Is Nothing Then
                    list.SelectedIndex = -1
                    item.Selected = True
                End If
            End If

        End Sub    'LoadTimeZoneDropDownList

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Localizes ModuleControl Titles
        ''' </summary>
        ''' <param name="ControlTitle">Current control title</param>
        ''' <param name="ControlSrc">Control Source</param>
        ''' <param name="Key">Control Key</param>
        ''' <returns>
        ''' Localized control title if found
        ''' </returns>
        ''' <remarks>
        ''' Resource keys are: ControlTitle_[key].Text
        ''' Key MUST be lowercase in the resource file
        ''' </remarks>
        ''' <history>
        ''' 	[vmasanas]	08/11/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function LocalizeControlTitle(ByVal ControlTitle As String, ByVal ControlSrc As String, ByVal Key As String) As String
            Dim reskey As String
            reskey = "ControlTitle_" + Key.ToLower + ".Text"
            Dim ResFile As String = ControlSrc.Substring(0, ControlSrc.LastIndexOf("/") + 1) + LocalResourceDirectory + ControlSrc.Substring(ControlSrc.LastIndexOf("/"), ControlSrc.LastIndexOf(".") - ControlSrc.LastIndexOf("/"))
            Dim localizedvalue As String = Services.Localization.Localization.GetString(reskey, ResFile)
            If Not localizedvalue Is Nothing Then
                Return localizedvalue
            Else
                Return ControlTitle
            End If
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LocalizeDataGrid creates localized Headers for a DataGrid
        ''' </summary>
        ''' <param name="grid">Grid t localize</param>
        ''' <param name="ResourceFile">The root name of the Resource File where the localized
        '''   text can be found</param>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub LocalizeDataGrid(ByRef grid As DataGrid, ByVal ResourceFile As String)

            Dim col As DataGridColumn
            Dim key As String
            Dim localizedText As String

            For Each col In grid.Columns
                'Localize Header Text
                key = col.HeaderText
                If key <> "" Then
                    localizedText = Services.Localization.Localization.GetString(key & ".Header", ResourceFile)
                    If localizedText <> "" Then
                        col.HeaderText = localizedText
                    End If
                End If

                'Localize Edit Text
                If TypeOf col Is EditCommandColumn Then
                    Dim editCol As EditCommandColumn = DirectCast(col, EditCommandColumn)
                    key = editCol.EditText
                    localizedText = Services.Localization.Localization.GetString(key & ".EditText", ResourceFile)
                    If localizedText <> "" Then
                        editCol.EditText = localizedText
                    End If
                End If

            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Localizes PortalSettings
        ''' </summary>
        ''' <remarks>
        ''' Localizes:
        ''' -DesktopTabs
        ''' -BreadCrumbs
        ''' Localized values are stored in httpcontext
        ''' </remarks>
        ''' <history>
        ''' 	[vmasanas]	11/08/2004	Created
        '''     [vmasanas]  12/09/2004  Localize ActiveTab
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub LocalizePortalSettings()
            Dim objPortalSettings As PortalSettings = CType(HttpContext.Current.Items("PortalSettings"), PortalSettings)

            objPortalSettings.DesktopTabs = LocalizeTabStripDetails(objPortalSettings.DesktopTabs)
            objPortalSettings.ActiveTab.BreadCrumbs = LocalizeTabStripDetails(objPortalSettings.ActiveTab.BreadCrumbs)
            If objPortalSettings.ActiveTab.IsAdminTab Then
                Dim strLocalizedTabName As String = Services.Localization.Localization.GetString(objPortalSettings.ActiveTab.TabName + ".String", Services.Localization.Localization.GlobalResourceFile)
                If strLocalizedTabName <> "" Then
                    objPortalSettings.ActiveTab.TabName = strLocalizedTabName
                    objPortalSettings.ActiveTab.Title = ""
                End If
            End If

            HttpContext.Current.Items("PortalSettings") = objPortalSettings
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Localizes the "Built In" Roles
        ''' </summary>
        ''' <remarks>
        ''' Localizes:
        ''' -DesktopTabs
        ''' -BreadCrumbs
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	02/01/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function LocalizeRole(ByVal role As String) As String

            Dim localRole As String

            Select Case role
                Case glbRoleAllUsersName, glbRoleSuperUserName, glbRoleUnauthUserName
                    Dim roleKey As String = role.Replace(" ", "")
                    localRole = GetString(roleKey)
                Case Else
                    localRole = role
            End Select

            Return localRole

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Does the replacement of tags on a givem message.
        ''' </summary>
        ''' <param name="MessageValue">The message to be formatted.</param>
        ''' <param name="Prefix">The prefix to look for <see cref="FormatSystemMessage" /> for supported tags.</param>
        ''' <param name="objObject">An object used to look for replacement values.</param>
        ''' <param name="objType">Object type for the object used to look for replacement values.</param>
        ''' <returns>The message with the found tags replaced.</returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Vicen]	05/07/2004	Documented
        '''     [cnurse]    10/06/2004  Moved from SystemMessages to Localization
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Shared Function PersonalizeSystemMessage(ByVal MessageValue As String, ByVal Prefix As String, ByVal objObject As Object, ByVal objType As Type) As String

            Dim intProperty As Integer
            Dim strPropertyName As String
            Dim strPropertyValue As String

            Dim objProperties As ArrayList = CBO.GetPropertyInfo(objType)

            For intProperty = 0 To objProperties.Count - 1
                strPropertyName = CType(objProperties(intProperty), PropertyInfo).Name
                If InStr(1, MessageValue, "[" & Prefix & strPropertyName & "]", CompareMethod.Text) <> 0 Then
                    Dim propInfo As PropertyInfo = CType(objProperties(intProperty), PropertyInfo)
                    Dim propValue As Object = propInfo.GetValue(objObject, Nothing)

                    If propValue Is Nothing = False Then
                        strPropertyValue = propValue.ToString()
                    End If

                    ' special case for encrypted passwords
                    If (Prefix & strPropertyName = "Membership:Password") And Convert.ToString(Common.Globals.HostSettings("EncryptionKey")) <> "" Then
                        Dim objSecurity As New PortalSecurity
                        strPropertyValue = objSecurity.Decrypt(Common.Globals.HostSettings("EncryptionKey").ToString, strPropertyValue)
                    End If

                    MessageValue = Replace(MessageValue, "[" & Prefix & strPropertyName & "]", strPropertyValue, , , CompareMethod.Text)
                End If
            Next intProperty

            Return MessageValue

        End Function

#End Region

    End Class

End Namespace
