'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Configuration
Imports System.Data
Imports System.Globalization
Imports System.IO
Imports System.Xml
Imports System.Xml.Schema
Imports ICSharpCode.SharpZipLib.Zip
Imports DotNetNuke.Modules.Admin.ResourceInstaller
Namespace DotNetNuke.Entities.Modules.Definitions
	Public Enum ModuleDefinitionVersion
		VUnknown = 0
		V1 = 1
		V2 = 2
		V2_Skin = 3
		V2_Provider = 4
        V3 = 5
        V3_1 = 6
	End Enum

    Public Class ModuleDefinitionValidator
        Inherits ResourceInstallerBase
        Dim _errs As New ArrayList
        Dim _xml As Stream
        Dim _schemaCollection As New XmlSchemaCollection

        Public Property Errors() As ArrayList
            Get
                Return _errs
            End Get
            Set(ByVal Value As ArrayList)
                _errs = Value
            End Set
        End Property

        Public ReadOnly Property XML() As String
            Get
                Dim xmlString As String
                Dim Buffer() As Byte

                Buffer = New Byte(Convert.ToInt32(_xml.Length) - 1) {}
                Dim size As Integer = 0
                While size < Buffer.Length
                    size += _xml.Read(Buffer, size, Buffer.Length - size)
                End While
                If size <> Buffer.Length Then
                    Throw New Exception(String.Format(Services.Localization.Localization.GetString("ReadError"), Buffer.Length, size))
                End If
                xmlString = System.Text.Encoding.ASCII.GetString(Buffer)
                System.Diagnostics.Debug.WriteLine(xmlString)

                Return xmlString
            End Get
        End Property

        Public ReadOnly Property SchemaCollection() As XmlSchemaCollection
            Get
                Return _schemaCollection
            End Get
        End Property

        Public Sub LoadXML(ByVal XmlStream As Stream)
            _xml = XmlStream
        End Sub

        Public Sub LoadSchema(ByVal Schema As String)
            _schemaCollection.Add("", Schema)
        End Sub

        Public Function ValidateDNN(ByVal XmlStream As Stream) As Boolean
            _xml = XmlStream
            _schemaCollection.Add("", GetDnnSchemaPath(_xml))
            Return IsValid()
        End Function

        Public Function IsValid() As Boolean
            'There is a bug here which I haven't been able to fix.
            'If the XML Instance does not include a reference to the
            'schema, then the validation fails.  If the reference exists
            'the the validation works correctly.

            'Create a validating reader
            Dim vreader As XmlValidatingReader
            '                        vreader = New XmlValidatingReader(_xml, XmlNodeType.Document,, Nothing)
            _xml.Seek(0, SeekOrigin.Begin)
            Dim xmlReader As New XmlTextReader(_xml)
            vreader = New XmlValidatingReader(xmlReader)

            vreader.Schemas.Add(_schemaCollection)

            vreader.ValidationType = ValidationType.Schema

            'Set the validation event handler.
            AddHandler vreader.ValidationEventHandler, New ValidationEventHandler(AddressOf ValidationCallBack)

            'Read and validate the XML data.
            While vreader.Read()
            End While

            'Close the reader.
            vreader.Close()

            Return (_errs.Count = 0)
        End Function
        Protected Sub ValidationCallBack(ByVal sender As Object, ByVal args As ValidationEventArgs)
            _errs.Add(args.Message)
        End Sub
        Public Function GetDnnSchemaPath(ByVal xmlStream As Stream) As String
            Dim Version As ModuleDefinitionVersion = GetModuleDefinitionVersion(xmlStream)
            Dim schemaPath As String

            Select Case Version
                Case ModuleDefinitionVersion.V2
                    schemaPath = "components\ResourceInstaller\ModuleDef_V2.xsd"
                Case ModuleDefinitionVersion.V3
                    schemaPath = "components\ResourceInstaller\ModuleDef_V3.xsd"
                Case ModuleDefinitionVersion.V3_1
                    schemaPath = "components\ResourceInstaller\ModuleDef_V31.xsd"
                Case ModuleDefinitionVersion.V2_Skin
                    schemaPath = "components\ResourceInstaller\ModuleDef_V2Skin.xsd"
                Case ModuleDefinitionVersion.V2_Provider
                    schemaPath = "components\ResourceInstaller\ModuleDef_V2Provider.xsd"
                Case ModuleDefinitionVersion.VUnknown
                    Throw New Exception(EXCEPTION_Format)
            End Select
            Return Path.Combine(Common.Globals.ApplicationMapPath, schemaPath)
        End Function
        Public Function GetModuleDefinitionVersion(ByVal xmlStream As Stream) As ModuleDefinitionVersion
            xmlStream.Seek(0, SeekOrigin.Begin)
            Dim xmlReader As New XmlTextReader(xmlStream)
            xmlReader.MoveToContent()

            'This test assumes provides a simple validation 
            Select Case xmlReader.LocalName.ToLower
                Case "module"
                    Return ModuleDefinitionVersion.V1
                Case "dotnetnuke"
                    Select Case xmlReader.GetAttribute("type")
                        Case "Module"
                            Select Case xmlReader.GetAttribute("version")
                                Case "2.0"
                                    Return ModuleDefinitionVersion.V2
                                Case "3.0"
                                    Return ModuleDefinitionVersion.V3
                                Case "3.1"
                                    Return ModuleDefinitionVersion.V3_1
                            End Select
                        Case "SkinObject"
                            Return ModuleDefinitionVersion.V2_Skin
                        Case "Provider"
                            Return ModuleDefinitionVersion.V2_Provider
                        Case Else
                            Return ModuleDefinitionVersion.VUnknown
                    End Select
                Case Else
                    Return ModuleDefinitionVersion.VUnknown
            End Select

        End Function


    End Class

End Namespace
