'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Configuration
Imports System.Data
Imports System.IO
Imports DotNetNuke.Entities.Modules.Actions

Namespace DotNetNuke.Entities.Modules

	''' -----------------------------------------------------------------------------
	''' Project	 : DotNetNuke
	''' Class	 : PortalModuleBase
	''' 
	''' -----------------------------------------------------------------------------
	''' <summary>
	''' The PortalModuleBase class defines a custom base class inherited by all
	''' desktop portal modules within the Portal.
	''' 
	''' The PortalModuleBase class defines portal specific properties
	''' that are used by the portal framework to correctly display portal modules
	''' </summary>
	''' <remarks>
	''' </remarks>
	''' <history>
	'''		[cnurse]	09/17/2004	Added Documentation
	'''								Modified LocalResourceFile to be Writeable
	'''		[cnurse]	10/21/2004	Modified Settings property to get both
	'''								TabModuleSettings and ModuleSettings
	''' </history>
	''' -----------------------------------------------------------------------------
	Public Class PortalModuleBase
		Inherits UserControl

#Region "Private Members"

		Private _isEditable As Integer = 0
		Private _localResourceFile As String
        Private _moduleConfiguration As ModuleInfo
		Private _settings As Hashtable
		Private _helpfile As String
		Private _helpurl As String
		Private _actions As ModuleActionCollection
        Private _cachedOutput As String = ""

#End Region

#Region "Public Properties"

		<Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
		Public Property Actions() As ModuleActionCollection

			Get
				If _actions Is Nothing Then
					_actions = New ModuleActionCollection
				End If
				Return _actions
			End Get
			Set(ByVal Value As ModuleActionCollection)
				_actions = Value
			End Set

        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheDirectory property is used to return the location of the "Cache"
        ''' Directory for the Module
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        '''   [cnurse] 04/28/2005  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheDirectory() As String
            Get
                Return PortalSettings.HomeDirectoryMapPath & "Cache"
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheFileName property is used to store the FileName for this Module's
        ''' Cache
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        '''   [cnurse] 04/28/2005  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheFileName() As String
            Get
                Return CacheDirectory & "\" & CleanFileName(CacheKey) & ".htm"
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheKey property is used to calculate a "unique" cache key
        ''' entry to be used to store/retrieve the portal module's content
        ''' from the ASP.NET Cache. Note that cache key allows two versions of the module
        ''' content to be stored - one for anonymous and one for authenticated users.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        '''   [cnurse] 17/9/2004  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheKey() As String

            Get
                Dim strCacheKey As String = "TabModule:"
                strCacheKey += TabModuleId.ToString & ":"
                strCacheKey += System.Threading.Thread.CurrentThread.CurrentCulture.ToString
                Return strCacheKey
            End Get

        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CacheMethod property is used to store the Method used for this Module's
        ''' Cache
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        '''   [cnurse] 04/28/2005  Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CacheMethod() As String
            Get
                Return Convert.ToString(Common.Globals.HostSettings("ModuleCaching"))
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl() As String

            Return EditUrl("", "", "Edit")

        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal ControlKey As String) As String

            Return EditUrl("", "", ControlKey)

        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal KeyName As String, ByVal KeyValue As String) As String

            Return EditUrl(KeyName, KeyValue, "Edit")

        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal KeyName As String, ByVal KeyValue As String, ByVal ControlKey As String) As String

            Dim key As String = ControlKey

            If (key = "") Then
                key = "Edit"
            End If

            If KeyName <> "" And KeyValue <> "" Then
                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, "mid=" & ModuleId.ToString, KeyName & "=" & KeyValue)
            Else
                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, "mid=" & ModuleId.ToString)
            End If

        End Function

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Function EditUrl(ByVal KeyName As String, ByVal KeyValue As String, ByVal ControlKey As String, ByVal ParamArray AdditionalParameters As String()) As String

            Dim key As String = ControlKey

            If (key = "") Then
                key = "Edit"
            End If

            If KeyName <> "" And KeyValue <> "" Then

                Dim params(AdditionalParameters.Length + 1) As String

                params(0) = "mid=" & ModuleId.ToString
                params(1) = KeyName & "=" & KeyValue

                For i As Integer = 0 To AdditionalParameters.Length - 1

                    params(i + 2) = AdditionalParameters(i)

                Next

                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, params)

            Else

                Dim params(AdditionalParameters.Length) As String

                params(0) = "mid=" & ModuleId.ToString

                For i As Integer = 0 To AdditionalParameters.Length - 1

                    params(i + 1) = AdditionalParameters(i)

                Next

                Return NavigateURL(PortalSettings.ActiveTab.TabID, key, params)

            End If

        End Function

        <ObsoleteAttribute("The HelpFile() property was deprecated in version 2.2. Help files are now stored in the /App_LocalResources folder beneath the module with the following resource key naming convention: ModuleHelp.Text")> _
  Public Property HelpFile() As String

            Get
                Return _helpfile
            End Get
            Set(ByVal Value As String)
                _helpfile = Value
            End Set

        End Property

        Public Property HelpURL() As String

            Get
                Return _helpurl
            End Get
            Set(ByVal Value As String)
                _helpurl = Value
            End Set

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property IsEditable() As Boolean

            Get

                ' Perform tri-state switch check to avoid having to perform a security
                ' role lookup on every property access (instead caching the result)
                If _isEditable = 0 Then

                    Dim blnPreview As Boolean = False
                    If Not Request.Cookies("_Tab_Admin_Preview" & PortalId.ToString) Is Nothing Then
                        blnPreview = Boolean.Parse(Request.Cookies("_Tab_Admin_Preview" & PortalId.ToString).Value)
                    End If
                    If PortalSettings.ActiveTab.ParentId = PortalSettings.AdminTabId Or PortalSettings.ActiveTab.ParentId = PortalSettings.SuperTabId Then
                        blnPreview = False
                    End If

                    Dim blnHasModuleEditPermissions As Boolean = _
                        (PortalSecurity.IsInRoles(_moduleConfiguration.AuthorizedEditRoles) = True) Or _
                        (PortalSecurity.IsInRoles(PortalSettings.ActiveTab.AdministratorRoles) = True) Or _
                        (PortalSecurity.IsInRoles(PortalSettings.AdministratorRoleName) = True)

                    If blnPreview = False And blnHasModuleEditPermissions = True Then
                        _isEditable = 1
                    Else
                        _isEditable = 2
                    End If
                End If

                Return _isEditable = 1
            End Get

        End Property

        Public Property LocalResourceFile() As String

            Get
                Dim fileRoot As String

                If _localResourceFile = "" Then
                    fileRoot = Me.TemplateSourceDirectory & "/" & Services.Localization.Localization.LocalResourceDirectory & "/" & Me.ID
                Else
                    fileRoot = _localResourceFile
                End If
                Return fileRoot
            End Get
            Set(ByVal Value As String)
                _localResourceFile = Value
            End Set

        End Property

        Public ReadOnly Property ModulePath() As String
            Get
                Return Me.TemplateSourceDirectory & "/"
            End Get
        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Property ModuleConfiguration() As ModuleInfo

            Get
                Return _moduleConfiguration
            End Get
            Set(ByVal Value As ModuleInfo)
                _moduleConfiguration = Value
            End Set

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property PortalId() As Integer

            Get
                Return PortalSettings.PortalId
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property TabId() As Integer

            Get
                If Not _moduleConfiguration Is Nothing Then
                    Return Convert.ToInt32(_moduleConfiguration.TabID)
                Else
                    Return Null.NullInteger
                End If
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property TabModuleId() As Integer

            Get
                If Not _moduleConfiguration Is Nothing Then
                    Return Convert.ToInt32(_moduleConfiguration.TabModuleID)
                Else
                    Return Null.NullInteger
                End If
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property ModuleId() As Integer

            Get
                If Not _moduleConfiguration Is Nothing Then
                    Return Convert.ToInt32(_moduleConfiguration.ModuleID)
                Else
                    Return Null.NullInteger
                End If
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property UserInfo() As UserInfo

            Get
                UserInfo = UserController.GetCurrentUserInfo
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property UserId() As Integer

            Get
                If HttpContext.Current.Request.IsAuthenticated Then
                    UserId = UserInfo.UserID
                Else
                    UserId = -1
                End If
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property PortalAlias() As PortalAliasInfo

            Get
                Return PortalSettings.PortalAlias
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property PortalSettings() As PortalSettings

            Get
                PortalSettings = PortalController.GetCurrentPortalSettings
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property Settings() As Hashtable

            Get

                If _settings Is Nothing Then

                    'Get Module Settings
                    _settings = PortalSettings.GetModuleSettings(ModuleId)

                    'Add TabModule Settings
                    _settings = PortalSettings.GetTabModuleSettings(TabModuleId, _settings)

                End If

                Return _settings
            End Get

        End Property

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property ContainerControl() As Control
            Get
                Return FindControlRecursive(Me, "ctr" & ModuleId.ToString)
            End Get
        End Property

#End Region

#Region "Public Methods"

        Public Sub SynchronizeModule()
            If CacheMethod <> "D" Then
                DataCache.RemoveCache(CacheKey)
            Else ' disk caching
                If System.IO.File.Exists(CacheFileName) Then
                    Try
                        System.IO.File.Delete(CacheFileName)
                    Catch
                        ' error deleting file from disk
                    End Try
                End If
            End If
        End Sub

        Public Function HasModulePermission(ByVal PermissionKey As String) As Boolean
            Return Security.Permissions.ModulePermissionController.HasModulePermission(Me.ModuleConfiguration.ModulePermissions, PermissionKey)
        End Function

#End Region

#Region "Protected Methods"

        Protected Function GetNextActionID() As Integer
            Return Actions.Count + 1
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The CreateChildControls method is called when the ASP.NET Page Framework
        ''' determines that it is time to instantiate a server control.
        ''' This method and attempts to resolve any previously cached output of the portal 
        ''' module from the ASP.NET cache.  
        ''' If it doesn't find cached output from a previous request, then it will instantiate 
        ''' and add the portal modules UserControl instance into the page tree.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        '''   [cnurse] 17/9/2004  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub CreateChildControls()

            If Not _moduleConfiguration Is Nothing Then

                ' if not in admin mode
                If PortalSecurity.IsInRole(PortalSettings.AdministratorRoleName.ToString) = False And PortalSecurity.IsInRoles(PortalSettings.ActiveTab.AdministratorRoles.ToString) = False Then

                    ' Attempt to resolve previously cached content 
                    If _moduleConfiguration.CacheTime <> 0 Then
                        If CacheMethod <> "D" Then
                            _cachedOutput = Convert.ToString(DataCache.GetCache(CacheKey))
                        Else ' cache from disk
                            If File.Exists(CacheFileName) Then
                                Dim cacheFile As New FileInfo(CacheFileName)
                                If cacheFile.CreationTime.AddSeconds(_moduleConfiguration.CacheTime) >= Now Then
                                    'Load from Cache
                                    Dim objStreamReader As StreamReader
                                    objStreamReader = cacheFile.OpenText()
                                    _cachedOutput = objStreamReader.ReadToEnd
                                    objStreamReader.Close()
                                Else
                                    'Cache Expired so delete it
                                    cacheFile.Delete()
                                End If
                            End If
                        End If
                    End If

                    ' If no cached content is found, then instantiate and add the portal
                    ' module user control into the portal's page server control tree
                    If _cachedOutput = "" And _moduleConfiguration.CacheTime > 0 Then

                        MyBase.CreateChildControls()

                        Dim objPortalModuleBase As Entities.Modules.PortalModuleBase = CType(Page.LoadControl(_moduleConfiguration.ControlSrc), PortalModuleBase)
                        objPortalModuleBase.ModuleConfiguration = Me.ModuleConfiguration

                        ' In skin.vb, the call to Me.Controls.Add(objPortalModuleBase) calls CreateChildControls() therefore
                        ' we need to indicate the control has already been created. We will manipulate the CacheTime property for this purpose. 
                        objPortalModuleBase.ModuleConfiguration.CacheTime = -(objPortalModuleBase.ModuleConfiguration.CacheTime)

                        Me.Controls.Add(objPortalModuleBase)

                    Else
                        ' restore the CacheTime property in preparation for the Render() event
                        If _moduleConfiguration.CacheTime < 0 Then
                            _moduleConfiguration.CacheTime = -(_moduleConfiguration.CacheTime)
                        End If
                    End If

                End If
            End If

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The Render method is called when the ASP.NET Page Framework
        ''' determines that it is time to render content into the page output stream.
        ''' This method and captures the output generated by the portal module user control
        ''' It then adds this content into the ASP.NET Cache for future requests.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        '''   [cnurse] 17/9/2004  Added Documentation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub Render(ByVal output As HtmlTextWriter)

            If Not _moduleConfiguration Is Nothing Then

                ' If no caching is specified or in admin mode, render the child tree and return 
                If _moduleConfiguration.CacheTime = 0 Or (PortalSecurity.IsInRoles(PortalSettings.AdministratorRoleName) = True Or PortalSecurity.IsInRoles(PortalSettings.ActiveTab.AdministratorRoles.ToString) = True) Then

                    MyBase.Render(output)

                Else 'output caching enabled

                    ' If no cached output was found from a previous request, render
                    ' child controls into a TextWriter, and then cache the results
                    If _cachedOutput = "" Then

                        Dim tempWriter As StringWriter = New StringWriter
                        MyBase.Render(New HtmlTextWriter(tempWriter))
                        _cachedOutput = tempWriter.ToString()

                        If CacheMethod <> "D" Then
                            DataCache.SetCache(CacheKey, _cachedOutput, DateTime.Now.AddSeconds(_moduleConfiguration.CacheTime))
                        Else ' cache to disk
                            Try
                                If Not Directory.Exists(CacheDirectory) Then
                                    Directory.CreateDirectory(CacheDirectory)
                                End If

                                Dim objStream As StreamWriter
                                objStream = File.CreateText(CacheFileName)
                                objStream.WriteLine(_cachedOutput)
                                objStream.Close()
                            Catch
                                ' error writing to disk
                            End Try
                        End If
                    End If

                    ' Output the user control's content
                    output.Write(_cachedOutput)
                End If

            Else

                MyBase.Render(output)

            End If

        End Sub

#End Region

    End Class

End Namespace
