'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Configuration
Imports System.Data
Imports System.IO
Imports DotNetNuke.Entities.Tabs
Imports DotNetNuke.Services.Cache
Imports System.Web.Caching

Namespace DotNetNuke.Common.Utilities

    Public Enum CoreCacheType
        Host = 1
        Portal = 2
        Tab = 3
    End Enum

    Public Class DataCache

        Public Shared ReadOnly Property CachePersistenceEnabled() As Boolean
            Get
                If Not System.Configuration.ConfigurationSettings.AppSettings("EnableCachePersistence") Is Nothing Then
                    If System.Configuration.ConfigurationSettings.AppSettings("EnableCachePersistence") = "true" Then
                        Return True
                    Else
                        Return False
                    End If
                Else
                    Return False
                End If
            End Get
        End Property

        Public Shared Function GetCache(ByVal CacheKey As String) As Object

            Return CachingProvider.Instance.GetItem(CacheKey)

        End Function

        Public Shared Function GetPersistentCacheItem(ByVal CacheKey As String, ByVal objType As Type) As Object

            Return CachingProvider.Instance.GetPersistentCacheItem(CacheKey, objType)

        End Function

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object)

            SetCache(CacheKey, objObject, False)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal PersistAppRestart As Boolean)

            CachingProvider.Instance.Insert(CacheKey, objObject, PersistAppRestart)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal objDependency As CacheDependency)

            SetCache(CacheKey, objObject, objDependency, False)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal objDependency As CacheDependency, ByVal PersistAppRestart As Boolean)

            CachingProvider.Instance.Insert(CacheKey, objObject, objDependency, PersistAppRestart)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal objDependency As CacheDependency, ByVal AbsoluteExpiration As Date, ByVal SlidingExpiration As System.TimeSpan)

            SetCache(CacheKey, objObject, objDependency, AbsoluteExpiration, SlidingExpiration, False)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal objDependency As CacheDependency, ByVal AbsoluteExpiration As Date, ByVal SlidingExpiration As System.TimeSpan, ByVal PersistAppRestart As Boolean)

            CachingProvider.Instance.Insert(CacheKey, objObject, objDependency, AbsoluteExpiration, SlidingExpiration, PersistAppRestart)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal SlidingExpiration As TimeSpan)

            SetCache(CacheKey, objObject, SlidingExpiration, False)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal SlidingExpiration As TimeSpan, ByVal PersistAppRestart As Boolean)

            CachingProvider.Instance.Insert(CacheKey, objObject, Nothing, Cache.NoAbsoluteExpiration, SlidingExpiration, PersistAppRestart)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal objDependency As CacheDependency, ByVal AbsoluteExpiration As Date, ByVal SlidingExpiration As System.TimeSpan, ByVal Priority As CacheItemPriority, ByVal OnRemoveCallback As CacheItemRemovedCallback)

            SetCache(CacheKey, objObject, objDependency, AbsoluteExpiration, SlidingExpiration, Priority, OnRemoveCallback, False)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal objDependency As CacheDependency, ByVal AbsoluteExpiration As Date, ByVal SlidingExpiration As System.TimeSpan, ByVal Priority As CacheItemPriority, ByVal OnRemoveCallback As CacheItemRemovedCallback, ByVal PersistAppRestart As Boolean)

            CachingProvider.Instance.Insert(CacheKey, objObject, Nothing, Cache.NoAbsoluteExpiration, SlidingExpiration, PersistAppRestart)

        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal AbsoluteExpiration As Date)

            SetCache(CacheKey, objObject, AbsoluteExpiration, False)


        End Sub

        Public Shared Sub SetCache(ByVal CacheKey As String, ByVal objObject As Object, ByVal AbsoluteExpiration As Date, ByVal PersistAppRestart As Boolean)

            CachingProvider.Instance.Insert(CacheKey, objObject, Nothing, AbsoluteExpiration, Cache.NoSlidingExpiration, PersistAppRestart)

        End Sub

        Public Shared Sub RemoveCache(ByVal CacheKey As String)

            CachingProvider.Instance.Remove(CacheKey)

        End Sub

        Public Shared Sub RemovePersistentCacheItem(ByVal CacheKey As String)

            CachingProvider.Instance.RemovePersistentCacheItem(CacheKey)

        End Sub



        <Obsolete("This method is obsolete. Use the new specific methods: ClearHostCache, ClearPortalCache, ClearTabCache.")> _
        Public Shared Sub ClearCoreCache(ByVal Type As CoreCacheType, Optional ByVal ID As Integer = -1, Optional ByVal Cascade As Boolean = False)

            Select Case Type
                Case CoreCacheType.Host
                    ClearHostCache(Cascade)
                Case CoreCacheType.Portal
                    ClearPortalCache(ID, Cascade)
                Case CoreCacheType.Tab
                    Dim objTab As TabInfo
                    Dim objTabs As New TabController
                    objTab = objTabs.GetTab(ID)
                    If Not objTab Is Nothing Then
                        ClearTabCache(ID, objTab.PortalID)
                    End If
            End Select

        End Sub

        Public Shared Sub ClearHostCache(ByVal Cascade As Boolean)

            RemoveCache("GetHostSettings")

            RemoveCache("GetPortalByAlias")

            RemoveCache("CSS")

            If Cascade Then
                Dim objPortals As New PortalController
                Dim objPortal As PortalInfo
                Dim arrPortals As ArrayList = objPortals.GetPortals

                Dim intIndex As Integer
                For intIndex = 0 To arrPortals.Count - 1
                    objPortal = CType(arrPortals(intIndex), PortalInfo)
                    ClearPortalCache(objPortal.PortalID, Cascade)
                Next
            End If

        End Sub

        Public Shared Sub ClearPortalCache(ByVal PortalId As Integer, ByVal Cascade As Boolean)

            Dim arrTabs As ArrayList = CType(GetCache("GetTabs" & PortalId.ToString), ArrayList)

            RemovePersistentCacheItem("GetPortalSettings" & PortalId.ToString)

            RemoveCache("GetTabs" & PortalId.ToString)

            If Cascade Then
                Dim objTab As TabInfo

                If arrTabs Is Nothing Then
                    Dim objTabs As New TabController
                    arrTabs = objTabs.GetTabs(PortalId)
                End If

                Dim intIndex As Integer
                For intIndex = 0 To arrTabs.Count - 1
                    objTab = CType(arrTabs(intIndex), TabInfo)
                    ClearTabCache(objTab.TabID)
                Next

                RemoveCache("GetTabPermissionsByPortal" & PortalId.ToString)
            End If

        End Sub

        Public Shared Sub ClearTabCache(ByVal TabId As Integer, ByVal PortalId As Integer)

            ClearTabCache(TabId)

            RemoveCache("GetTabPermissionsByPortal" & PortalId.ToString)

        End Sub

        Private Shared Sub ClearTabCache(ByVal TabId As Integer)

            RemoveCache("GetTab" & TabId.ToString)

            RemoveCache("GetPortalTabModules" & TabId.ToString)

        End Sub

    End Class

End Namespace