'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Drawing
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.IO
Imports System.Collections
Imports System.Data
Imports DotNetNuke.Security.Roles
Imports DotNetNuke.Services.Localization

Namespace DotNetNuke.Security.Permissions.Controls

    Public Class FolderPermissionsGrid
        Inherits PermissionsGrid

        Private _roles As ArrayList
        Private _Permissions As ArrayList
        Private _FolderPermissions As ArrayList

        Private _FolderPath As String = ""

        Public Property FolderPath() As String
            Get
                If _FolderPath <> "" Then
                    Return _FolderPath
                Else
                    Return ViewState("FolderPath").ToString
                End If
            End Get
            Set(ByVal Value As String)
                _FolderPath = Value
                ViewState("FolderPath") = Value
            End Set
        End Property

        Public ReadOnly Property Permissions() As Security.Permissions.FolderPermissionCollection
            Get
                Dim _portalSettings As PortalSettings = PortalController.GetCurrentPortalSettings
                Dim arrFolderPermissions As New Security.Permissions.FolderPermissionCollection
                Dim objRoleController As New RoleController

                Dim objFolderPermission As Security.Permissions.FolderPermissionInfo
                Dim dgi As DataGridItem
                For Each dgi In Items
                    Dim i As Integer
                    For i = 1 To dgi.Cells.Count - 1
                        'all except first cell which is role names
                        If dgi.Cells(i).Controls.Count > 0 Then
                            objFolderPermission = New Security.Permissions.FolderPermissionInfo
                            Dim cb As CheckBox = CType(dgi.Cells(i).Controls(0), CheckBox)
                            If cb.Checked Then
                                Dim Settings As String() = Split(cb.ID, "|")
                                objFolderPermission.PermissionID = Convert.ToInt32(Settings(1))
                                Dim objRoleInfo As RoleInfo
                                Select Case Settings(3)
                                    Case glbRoleAllUsersName
                                        objRoleInfo = New RoleInfo
                                        objRoleInfo.RoleID = Convert.ToInt32(glbRoleAllUsers)
                                        objRoleInfo.RoleName = glbRoleAllUsersName
                                        objRoleInfo.PortalID = _portalSettings.PortalId
                                    Case glbRoleUnauthUserName
                                        objRoleInfo = New RoleInfo
                                        objRoleInfo.RoleID = Convert.ToInt32(glbRoleUnauthUser)
                                        objRoleInfo.RoleName = glbRoleUnauthUserName
                                        objRoleInfo.PortalID = _portalSettings.PortalId
                                    Case Else
                                        objRoleInfo = objRoleController.GetRoleByName(_portalSettings.PortalId, Settings(3))
                                End Select

                                objFolderPermission.RoleID = objRoleInfo.RoleID
                                'If cb.Checked Then
                                objFolderPermission.AllowAccess = True
                                'Else
                                'objTabPermission.AllowAccess = False
                                'End If

                                If Settings(2) = "" Then
                                    objFolderPermission.FolderPermissionID = -1
                                Else
                                    objFolderPermission.FolderPermissionID = Convert.ToInt32(Settings(2))
                                End If
                                objFolderPermission.FolderPath = FolderPath
                                arrFolderPermissions.Add(objFolderPermission)
                            End If
                        End If
                    Next
                Next
                Return arrFolderPermissions
            End Get
        End Property

        Public Overrides Sub GenerateDataGrid()
            Columns.Clear()
            If FolderPath = "" Then
                FolderPath = viewstate("FolderPath").ToString
            End If
            SetPermissions()

            Dim textCol As New BoundColumn
            textCol.HeaderText = ""
            textCol.DataField = "Roles"
            Columns.Add(textCol)

            Dim checkCol As Controls.CheckBoxColumn
            Dim i As Integer
            For i = 0 To _Permissions.Count - 1
                Dim objPermission As Security.Permissions.PermissionInfo
                objPermission = CType(_Permissions(i), Security.Permissions.PermissionInfo)
                checkCol = New Controls.CheckBoxColumn
                Dim locName As String = Services.Localization.Localization.GetString(objPermission.PermissionName + ".Permission", Services.Localization.Localization.GlobalResourceFile)
                checkCol.HeaderText = IIf(locName <> "", locName, objPermission.PermissionName).ToString
                checkCol.DataField = objPermission.PermissionName
                Columns.Add(checkCol)
            Next
            DataSource = GetPermissionsDataTable()
            DataBind()
            Me.DynamicColumnAdded = True
        End Sub

        Private Function GetPermissionsDataTable() As DataTable
            Dim dt As New DataTable
            Dim col As DataColumn
            Dim i As Integer
            Try
                col = New DataColumn("Roles")
                dt.Columns.Add(col)
                For i = 0 To _Permissions.Count - 1
                    Dim objPerm As Security.Permissions.PermissionInfo
                    objPerm = CType(_Permissions(i), Security.Permissions.PermissionInfo)
                    col = New DataColumn(objPerm.PermissionName)
                    dt.Columns.Add(col)
                Next
            Catch ex As Exception
                Dim s As String = ex.Message
                s = s
            End Try

            Dim row As DataRow
            row = dt.NewRow

            For i = 0 To _roles.Count - 1
                Dim role As RoleInfo = DirectCast(_roles(i), RoleInfo)
                row = dt.NewRow
                row(0) = Localization.LocalizeRole(role.RoleName)

                Dim j As Integer
                For j = 0 To _Permissions.Count - 1
                    Dim objPerm As Security.Permissions.PermissionInfo
                    objPerm = CType(_Permissions(j), Security.Permissions.PermissionInfo)
                    Dim key As String
                    Dim CheckBoxEnabled As String = "Y"
                    Dim objFolderPermission As Security.Permissions.FolderPermissionInfo = FolderHasPermission(objPerm.PermissionID, _FolderPermissions, role.RoleName)
                    If Not objFolderPermission Is Nothing Then
                        If role.RoleID = PortalController.GetCurrentPortalSettings.AdministratorRoleId Then
                            CheckBoxEnabled = "N"
                        End If
                        key = "|" + Convert.ToString(objPerm.PermissionID) + "|" + Convert.ToString(objFolderPermission.FolderPermissionID) + "|" + role.RoleName + "|" + role.RoleID.ToString + "|" + CheckBoxEnabled
                        If objFolderPermission.AllowAccess Or CheckBoxEnabled = "N" Then
                            row(j + 1) = "True" + key
                        Else
                            row(j + 1) = "False" + key
                        End If
                    Else
                        If role.RoleID = PortalController.GetCurrentPortalSettings.AdministratorRoleId Then
                            CheckBoxEnabled = "N"
                        End If
                        key = "|" + Convert.ToString(objPerm.PermissionID) + "||" + role.RoleName + "|" + role.RoleID.ToString + "|" + CheckBoxEnabled
                        row(j + 1) = "False" + key
                    End If
                Next
                dt.Rows.Add(row)
            Next
            Return dt
        End Function

        Private Function FolderHasPermission(ByVal PermissionID As Integer, ByVal arrFolderPermissions As ArrayList, ByVal RoleName As String) As Security.Permissions.FolderPermissionInfo
            Dim i As Integer
            For i = 0 To arrFolderPermissions.Count - 1
                Dim objFolderPermission As Security.Permissions.FolderPermissionInfo = CType(arrFolderPermissions(i), Security.Permissions.FolderPermissionInfo)
                If objFolderPermission.RoleName = RoleName And PermissionID = objFolderPermission.PermissionID Then
                    Return objFolderPermission
                End If
            Next
            Return Nothing
        End Function

        Private Sub SetPermissions()
            ' Obtain PortalSettings from Current Context
            Dim _portalSettings As PortalSettings = PortalController.GetCurrentPortalSettings
            Dim objRoleController As New RoleController
            _roles = objRoleController.GetPortalRoles(_portalSettings.PortalId)

            Dim r As New RoleInfo
            r.RoleID = Integer.Parse(glbRoleUnauthUser)
            r.RoleName = glbRoleUnauthUserName
            _roles.Add(r)
            r = New RoleInfo
            r.RoleID = Integer.Parse(glbRoleAllUsers)
            r.RoleName = glbRoleAllUsersName
            _roles.Add(r)
            _roles.Reverse()
            _roles.Sort(New RoleComparer)

            Dim objFolderPermissionController As New Security.Permissions.FolderPermissionController
            Dim intPortalID As Integer
            If _portalSettings.ActiveTab.ParentId = _portalSettings.SuperTabId Then 'if we are in host filemanager then we need to pass a null portal id
                intPortalID = Null.NullInteger
            Else
                intPortalID = _portalSettings.PortalId
            End If
            _FolderPermissions = objFolderPermissionController.GetFolderPermissionsByFolder(intPortalID, Me.FolderPath)
            Dim objPermissionController As New Security.Permissions.PermissionController
            _Permissions = objPermissionController.GetPermissionsByFolder(intPortalID, Me.FolderPath)

        End Sub

    End Class


End Namespace