'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System
Imports System.Web.Caching
Imports System.Reflection
Imports System.Xml
Imports System.Xml.Serialization
Imports System.Text
Imports System.IO

Namespace DotNetNuke.Common.Utilities

	'*********************************************************************
	'
	' CBO Class
	'
    ' Class that hydrates custom business objects with data. Please
    ' note that this utility class can only be used on objects with "simple"
    ' data types. If the object contains "complex" data types such as 
    ' ArrayLists, HashTables, Custom Objects, etc... then the developer 
    ' will need to write custom code for the hydration of these "complex" 
    ' data types.
	'
	'*********************************************************************

	Public Class CBO

		Public Shared Function GetPropertyInfo(ByVal objType As Type) As ArrayList

			' Use the cache because the reflection used later is expensive
			Dim objProperties As ArrayList = CType(DataCache.GetCache(objType.FullName), ArrayList)

			If objProperties Is Nothing Then
				objProperties = New ArrayList
				Dim objProperty As PropertyInfo
				For Each objProperty In objType.GetProperties()
					objProperties.Add(objProperty)
				Next
				DataCache.SetCache(objType.FullName, objProperties)
			End If

			Return objProperties

		End Function

		Private Shared Function GetOrdinals(ByVal objProperties As ArrayList, ByVal dr As IDataReader) As Integer()

			Dim arrOrdinals(objProperties.Count) As Integer
			Dim intProperty As Integer

			If Not dr Is Nothing Then
				For intProperty = 0 To objProperties.Count - 1
					arrOrdinals(intProperty) = -1
					Try
						arrOrdinals(intProperty) = dr.GetOrdinal(CType(objProperties(intProperty), PropertyInfo).Name)
					Catch
						' property does not exist in datareader
					End Try
				Next intProperty
			End If

			Return arrOrdinals

		End Function

        Private Shared Function CreateObject(ByVal objType As Type, ByVal dr As IDataReader, ByVal objProperties As ArrayList, ByVal arrOrdinals As Integer()) As Object

            Dim objPropertyInfo As PropertyInfo
            Dim objValue As Object
            Dim objPropertyType As Type
            Dim intProperty As Integer

            Dim objObject As Object = Activator.CreateInstance(objType)

            ' fill object with values from datareader
            For intProperty = 0 To objProperties.Count - 1
                objPropertyInfo = CType(objProperties(intProperty), PropertyInfo)
                If objPropertyInfo.CanWrite Then
                    objValue = Null.SetNull(objPropertyInfo)
                    If arrOrdinals(intProperty) <> -1 Then
                        If IsDBNull(dr.GetValue(arrOrdinals(intProperty))) Then
                            ' translate Null value
                            objPropertyInfo.SetValue(objObject, objValue, Nothing)
                        Else
                            Try
                                ' try implicit conversion first
                                objPropertyInfo.SetValue(objObject, dr.GetValue(arrOrdinals(intProperty)), Nothing)
                            Catch
                                ' business object info class member data type does not match datareader member data type
                                Try
                                    objPropertyType = objPropertyInfo.PropertyType
                                    'need to handle enumeration conversions differently than other base types
                                    If objPropertyType.BaseType.Equals(GetType(System.Enum)) Then
                                        ' check if value is numeric and if not convert to integer ( supports databases like Oracle )
                                        If IsNumeric(dr.GetValue(arrOrdinals(intProperty))) Then
                                            CType(objProperties(intProperty), PropertyInfo).SetValue(objObject, System.Enum.ToObject(objPropertyType, Convert.ToInt32(dr.GetValue(arrOrdinals(intProperty)))), Nothing)
                                        Else
                                            CType(objProperties(intProperty), PropertyInfo).SetValue(objObject, System.Enum.ToObject(objPropertyType, dr.GetValue(arrOrdinals(intProperty))), Nothing)
                                        End If
                                    Else
                                        ' try explicit conversion
                                        objPropertyInfo.SetValue(objObject, Convert.ChangeType(dr.GetValue(arrOrdinals(intProperty)), objPropertyType), Nothing)
                                    End If
                                Catch
                                    objPropertyInfo.SetValue(objObject, Convert.ChangeType(dr.GetValue(arrOrdinals(intProperty)), objPropertyType), Nothing)
                                End Try
                            End Try
                        End If
                    Else
                        ' property does not exist in datareader
                    End If
                End If
            Next intProperty

            Return objObject

        End Function

        Public Shared Function FillObject(ByVal dr As IDataReader, ByVal objType As Type) As Object

            Return FillObject(dr, objType, True)

        End Function

        Public Shared Function FillObject(ByVal dr As IDataReader, ByVal objType As Type, ByVal ManageDataReader As Boolean) As Object

            Dim objFillObject As Object
            Dim intProperty As Integer

            ' get properties for type
            Dim objProperties As ArrayList = GetPropertyInfo(objType)

            ' get ordinal positions in datareader
            Dim arrOrdinals As Integer() = GetOrdinals(objProperties, dr)

            Dim Continue As Boolean
            If ManageDataReader Then
                Continue = False
                ' read datareader
                If dr.Read() Then
                    Continue = True
                End If
            Else
                Continue = True
            End If

            If Continue Then
                ' create custom business object
                objFillObject = CreateObject(objType, dr, objProperties, arrOrdinals)
            Else
                objFillObject = Nothing
            End If

            If ManageDataReader Then
                ' close datareader
                If Not dr Is Nothing Then
                    dr.Close()
                End If
            End If

            Return objFillObject

        End Function

        Public Shared Function FillCollection(ByVal dr As IDataReader, ByVal objType As Type) As ArrayList

            Dim objFillCollection As New ArrayList
            Dim objFillObject As Object
            Dim intProperty As Integer

            ' get properties for type
            Dim objProperties As ArrayList = GetPropertyInfo(objType)

            ' get ordinal positions in datareader
            Dim arrOrdinals As Integer() = GetOrdinals(objProperties, dr)

            ' iterate datareader
            While dr.Read
                ' fill business object
                objFillObject = CreateObject(objType, dr, objProperties, arrOrdinals)
                ' add to collection
                objFillCollection.Add(objFillObject)
            End While

            ' close datareader
            If Not dr Is Nothing Then
                dr.Close()
            End If

            Return objFillCollection

        End Function

        Public Shared Function FillCollection(ByVal dr As IDataReader, ByVal objType As Type, ByRef objToFill As IList) As IList

            Dim objFillObject As Object
            Dim intProperty As Integer

            ' get properties for type
            Dim objProperties As ArrayList = GetPropertyInfo(objType)

            ' get ordinal positions in datareader
            Dim arrOrdinals As Integer() = GetOrdinals(objProperties, dr)

            ' iterate datareader
            While dr.Read
                ' fill business object
                objFillObject = CreateObject(objType, dr, objProperties, arrOrdinals)
                ' add to collection
                objToFill.Add(objFillObject)
            End While

            ' close datareader
            If Not dr Is Nothing Then
                dr.Close()
            End If

            Return objToFill

        End Function

        Public Shared Function InitializeObject(ByVal objObject As Object, ByVal objType As Type) As Object

            Dim objPropertyInfo As PropertyInfo
            Dim objValue As Object
            Dim intProperty As Integer

            ' get properties for type
            Dim objProperties As ArrayList = GetPropertyInfo(objType)

            ' initialize properties
            For intProperty = 0 To objProperties.Count - 1
                objPropertyInfo = CType(objProperties(intProperty), PropertyInfo)
                If objPropertyInfo.CanWrite Then
                    objValue = Null.SetNull(objPropertyInfo)
                    objPropertyInfo.SetValue(objObject, objValue, Nothing)
                End If
            Next intProperty

            Return objObject

        End Function



        Public Shared Function Serialize(ByVal objObject As Object) As XmlDocument

            Dim objXmlSerializer As New XmlSerializer(objObject.GetType())

            Dim objStringBuilder As New StringBuilder

            Dim objTextWriter As TextWriter = New StringWriter(objStringBuilder)

            objXmlSerializer.Serialize(objTextWriter, objObject)

            Dim objStringReader As New StringReader(objTextWriter.ToString())

            Dim objDataSet As New DataSet

            objDataSet.ReadXml(objStringReader)

            Dim xmlSerializedObject As New XmlDocument

            xmlSerializedObject.LoadXml(objDataSet.GetXml())

            Return xmlSerializedObject

        End Function

        Public Shared Function CloneObject(ByVal ObjectToClone As Object) As Object    'Implements System.ICloneable.Clone
            Try
                Dim newObject As Object = DotNetNuke.Framework.Reflection.CreateObject(ObjectToClone.GetType().AssemblyQualifiedName, ObjectToClone.GetType().AssemblyQualifiedName)

                Dim props As ArrayList = GetPropertyInfo(newObject.GetType())

                Dim i As Integer = 0

                For i = 0 To GetPropertyInfo(ObjectToClone.GetType()).Count - 1
                    Dim p As PropertyInfo = CType(GetPropertyInfo(ObjectToClone.GetType())(i), PropertyInfo)

                    Dim ICloneType As Type = p.PropertyType.GetInterface("ICloneable", True)
                    If CType(props(i), PropertyInfo).CanWrite Then
                        If Not (ICloneType Is Nothing) Then
                            Dim IClone As ICloneable = CType(p.GetValue(ObjectToClone, Nothing), ICloneable)
                            CType(props(i), PropertyInfo).SetValue(newObject, IClone.Clone(), Nothing)
                        Else
                            CType(props(i), PropertyInfo).SetValue(newObject, p.GetValue(ObjectToClone, Nothing), Nothing)
                        End If

                        Dim IEnumerableType As Type = p.PropertyType.GetInterface("IEnumerable", True)
                        If Not (IEnumerableType Is Nothing) Then
                            Dim IEnum As IEnumerable = CType(p.GetValue(ObjectToClone, Nothing), IEnumerable)

                            Dim IListType As Type = CType(props(i), PropertyInfo).PropertyType.GetInterface("IList", True)
                            Dim IDicType As Type = CType(props(i), PropertyInfo).PropertyType.GetInterface("IDictionary", True)

                            Dim j As Integer = 0
                            If Not (IListType Is Nothing) Then
                                Dim list As IList = CType(CType(props(i), PropertyInfo).GetValue(newObject, Nothing), IList)

                                Dim obj As Object
                                For Each obj In IEnum
                                    ICloneType = obj.GetType().GetInterface("ICloneable", True)

                                    If Not (ICloneType Is Nothing) Then
                                        Dim tmpClone As ICloneable = CType(obj, ICloneable)
                                        list(j) = tmpClone.Clone()
                                    End If

                                    j += 1
                                Next obj
                            Else
                                If Not (IDicType Is Nothing) Then
                                    Dim dic As IDictionary = CType(CType(props(i), PropertyInfo).GetValue(newObject, Nothing), IDictionary)
                                    j = 0

                                    Dim de As DictionaryEntry
                                    For Each de In IEnum

                                        ICloneType = de.Value.GetType().GetInterface("ICloneable", True)

                                        If Not (ICloneType Is Nothing) Then
                                            Dim tmpClone As ICloneable = CType(de.Value, ICloneable)
                                            dic(de.Key) = tmpClone.Clone()
                                        End If
                                        j += 1
                                    Next de
                                End If
                            End If
                        End If
                    End If
                Next
                Return newObject
            Catch exc As Exception
                LogException(exc)
            End Try
        End Function

    End Class

End Namespace