'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.ComponentModel
Imports System.Diagnostics
Imports System.Globalization
Imports System.Reflection
Imports System.Text.RegularExpressions
Imports System.Threading
Imports System.Web.UI
Imports System.Web.UI.HtmlControls
Imports System.Web.UI.WebControls

Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Entities.Portals

Namespace DotNetNuke.Framework

    Public MustInherit Class PageBase

        Inherits System.Web.UI.Page

#Region "Private Members"
        Private _localizedControls As ArrayList
        Private _localResourceFile As String

#End Region

#Region "Public Properties"

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
Public ReadOnly Property PortalSettings() As PortalSettings

            Get
                PortalSettings = PortalController.GetCurrentPortalSettings
            End Get

        End Property

        'Localization
        Public Property PageCulture() As CultureInfo
            Get
                Dim ci As CultureInfo
                Dim preferredLanguage As String

                'first try if a specific language is requested by cookie, querystring, or form
                If Not (HttpContext.Current Is Nothing) Then
                    Try
                        preferredLanguage = HttpContext.Current.Request("language")
                    Catch
                    End Try
                End If
                If preferredLanguage <> "" Then
                    Try
                        ci = New CultureInfo(preferredLanguage)
                        'save preferred language in cookie
                        SetLanguage(ci.Name)
                    Catch
                        ' the preferredlanguage could not be converted to a CultureInfo object
                    End Try
                End If

                If ci Is Nothing Then
                    ' next try to get the preferred language of the logged on user
                    Dim objUserInfo As UserInfo = UserController.GetCurrentUserInfo
                    If objUserInfo.UserID <> -1 Then
                        If objUserInfo.Profile.PreferredLocale <> "" Then
                            ci = New CultureInfo(objUserInfo.Profile.PreferredLocale)
                            SetLanguage("") 'clear cookie
                        End If
                    End If
                End If

                If ci Is Nothing Then
                    ' use Request.UserLanguages to get the preferred language.
                    ' the first entry is the most preferred language
                    preferredLanguage = ""
                    If Not (HttpContext.Current Is Nothing) Then
                        If Not (HttpContext.Current.Request.UserLanguages Is Nothing) Then
                            Try
                                preferredLanguage = HttpContext.Current.Request.UserLanguages(0)
                            Catch
                            End Try
                        End If
                    End If
                    If preferredLanguage.Trim <> "" Then
                        Try
                            ci = New CultureInfo(preferredLanguage)
                        Catch
                            ' the preferredlanguage could not be converted to a CultureInfo object
                        End Try
                    End If
                End If

                If ci Is Nothing Then
                    If PortalSettings.DefaultLanguage = "" Then
                        ci = New CultureInfo(Convert.ToString(Services.Localization.Localization.SystemLocale))
                    Else
                        ci = New CultureInfo(PortalSettings.DefaultLanguage)
                    End If
                End If

                ' Check if locale is still supported
                Dim myLocaleCollection As LocaleCollection = Services.Localization.Localization.GetSupportedLocales()
                If myLocaleCollection.Item(ci.Name) Is Nothing Then
                    Dim i As Integer
                    'get the first part of the locale in the collection, locale is of form xx-XX where xx = language
                    'this is to make sure that no locale is available with the same language as the prefered language of the anonymous user
                    'the prefered language might just be the language without the country (ie. en, nl, de)
                    'get the first part of the locale in the collection
                    'and check against the first part of the prefered locale
                    'this is to check whether the first part (xx) is part of the collection
                    'ie en-GB is not available, but en-US is, then new locale will be en-US
                    For i = 0 To myLocaleCollection.Count - 1
                        If Split(Convert.ToString(myLocaleCollection.Item(i).Key), "-")(0) = Split(ci.Name.ToString, "-")(0) Then
                            ci = New CultureInfo(Convert.ToString(myLocaleCollection.Item(i).Key))
                            Return ci
                            Exit Property
                        End If
                    Next
                    'No locale found in the collection, so revert back to system fallback locale
                    ci = New CultureInfo(Convert.ToString(Services.Localization.Localization.SystemLocale))
                End If

                Return ci

            End Get
            Set(ByVal Value As CultureInfo)
                Thread.CurrentThread.CurrentUICulture = PageCulture
                Thread.CurrentThread.CurrentCulture = PageCulture
            End Set
        End Property

        Public Property LocalResourceFile() As String
            Get
                Dim fileRoot As String
                Dim page As String() = Request.ServerVariables("SCRIPT_NAME").Split("/"c)

                If _localResourceFile = "" Then
                    fileRoot = Me.TemplateSourceDirectory & "/" & Services.Localization.Localization.LocalResourceDirectory & "/" & page(page.GetUpperBound(0)) & ".resx"
                Else
                    fileRoot = _localResourceFile
                End If
                Return fileRoot
            End Get
            Set(ByVal Value As String)
                _localResourceFile = Value
            End Set
        End Property

#End Region

#Region "Public Methods"
        ''' <summary>
        ''' <para>Constructor</para>
        ''' </summary>
        Public Sub New()
            _localizedControls = New ArrayList
        End Sub    'New

        Protected Overrides Sub OnInit(ByVal e As System.EventArgs)
            ' Set the current culture
            Thread.CurrentThread.CurrentUICulture = PageCulture
            Thread.CurrentThread.CurrentCulture = PageCulture
            ' Localize portalsettings
            Services.Localization.Localization.LocalizePortalSettings()
            MyBase.OnInit(e)
        End Sub

        Public Function HasTabPermission(ByVal PermissionKey As String) As Boolean
            Return Security.Permissions.TabPermissionController.HasTabPermission(PortalSettings.ActiveTab.TabPermissions, PermissionKey)
        End Function


        ''' <summary>
        ''' <para>ProcessControl peforms the high level localization for a single control and optionally it's children.</para>
        ''' </summary>
        ''' <param name="c">Control to find the AttributeCollection on</param>
        ''' <param name="affectedControls">ArrayList that hold the controls that have been localized. This is later used for the removal of the key attribute.</param>				
        ''' <param name="includeChildren">If true, causes this method to process children of this controls.</param>
        Friend Sub ProcessControl(ByVal c As Control, ByVal affectedControls As ArrayList, ByVal includeChildren As Boolean, ByVal ResourceFileRoot As String)
            ' Perform the substitution if a key was found			
            Dim key As String = GetControlAttribute(c, affectedControls)
            If Not key Is Nothing Then
                'Translation starts here ....
                Dim value As String
                value = Services.Localization.Localization.GetString(key, ResourceFileRoot)

                If TypeOf c Is Label Then
                    Dim ctrl As Label
                    ctrl = DirectCast(c, Label)
                    If value <> "" Then
                        ctrl.Text = value
                    End If
                End If
                If TypeOf c Is LinkButton Then
                    Dim ctrl As LinkButton
                    ctrl = DirectCast(c, LinkButton)
                    If value <> "" Then

                        Dim imgMatches As MatchCollection = Regex.Matches(value, "<(a|link|img|script|input|form).[^>]*(href|src|action)=(\""|'|)(.[^\""']*)(\""|'|)[^>]*>", RegexOptions.IgnoreCase)

                        For Each _match As Match In imgMatches

                            If (_match.Groups(_match.Groups.Count - 2).Value.IndexOf("~") <> -1) Then

                                Dim resolvedUrl As String = Page.ResolveUrl(_match.Groups(_match.Groups.Count - 2).Value)

                                value = value.Replace(_match.Groups(_match.Groups.Count - 2).Value, resolvedUrl)

                            End If

                        Next

                        ctrl.Text = value
                    End If
                End If
                If TypeOf c Is HyperLink Then
                    Dim ctrl As HyperLink
                    ctrl = DirectCast(c, HyperLink)
                    If value <> "" Then
                        ctrl.Text = value
                    End If
                End If
                If TypeOf c Is ImageButton Then
                    Dim ctrl As ImageButton
                    ctrl = DirectCast(c, ImageButton)
                    If value <> "" Then
                        ctrl.AlternateText = value
                    End If
                End If
                If TypeOf c Is Button Then
                    Dim ctrl As Button
                    ctrl = DirectCast(c, Button)
                    If value <> "" Then
                        ctrl.Text = value
                    End If
                End If
                If TypeOf c Is HtmlControls.HtmlImage Then
                    Dim ctrl As HtmlControls.HtmlImage
                    ctrl = DirectCast(c, HtmlControls.HtmlImage)
                    If value <> "" Then
                        ctrl.Alt = value
                    End If
                End If
                If TypeOf c Is CheckBox Then
                    Dim ctrl As CheckBox
                    ctrl = DirectCast(c, CheckBox)
                    If value <> "" Then
                        ctrl.Text = value
                    End If
                End If
                If TypeOf c Is BaseValidator Then
                    Dim ctrl As BaseValidator
                    ctrl = DirectCast(c, BaseValidator)
                    If value <> "" Then
                        ctrl.ErrorMessage = value
                    End If
                End If
                If TypeOf c Is System.Web.UI.WebControls.Image Then
                    Dim ctrl As System.Web.UI.WebControls.Image
                    ctrl = DirectCast(c, System.Web.UI.WebControls.Image)
                    If value <> "" Then
                        ctrl.AlternateText = value
                    End If
                End If
            End If

            ' Translate radiobuttonlist items here 
            If TypeOf c Is RadioButtonList Then
                Dim ctrl As RadioButtonList
                ctrl = DirectCast(c, RadioButtonList)
                Dim i As Integer
                For i = 0 To ctrl.Items.Count - 1
                    Dim ac As System.Web.UI.AttributeCollection = Nothing
                    ac = ctrl.Items(i).Attributes
                    key = ac(Services.Localization.Localization.KeyName)
                    If Not key Is Nothing Then
                        Dim value As String = Services.Localization.Localization.GetString(key, ResourceFileRoot)
                        If value <> "" Then
                            ctrl.Items(i).Text = value
                        End If
                    End If
                Next
            End If

            ' Translate dropdownlist items here 
            If TypeOf c Is DropDownList Then
                Dim ctrl As DropDownList
                ctrl = DirectCast(c, DropDownList)
                Dim i As Integer
                For i = 0 To ctrl.Items.Count - 1
                    Dim ac As System.Web.UI.AttributeCollection = Nothing
                    ac = ctrl.Items(i).Attributes
                    key = ac(Services.Localization.Localization.KeyName)
                    If Not key Is Nothing Then
                        Dim value As String = Services.Localization.Localization.GetString(key, ResourceFileRoot)
                        If value <> "" Then
                            ctrl.Items(i).Text = value
                        End If
                    End If
                Next
            End If

            '' UrlRewriting Issue - ResolveClientUrl gets called instead of ResolveUrl
            '' Manual Override to ResolveUrl
            If TypeOf c Is System.Web.UI.WebControls.Image Then
                Dim ctrl As System.web.UI.WebControls.Image
                ctrl = DirectCast(c, System.web.UI.WebControls.Image)
                If (ctrl.ImageUrl.IndexOf("~") <> -1) Then
                    ctrl.ImageUrl = Page.ResolveUrl(ctrl.ImageUrl)
                End If
            End If

            ' UrlRewriting Issue - ResolveClientUrl gets called instead of ResolveUrl
            ' Manual Override to ResolveUrl
            If TypeOf c Is HtmlControls.HtmlImage Then
                Dim ctrl As HtmlControls.HtmlImage
                ctrl = DirectCast(c, HtmlControls.HtmlImage)
                If (ctrl.Src.IndexOf("~") <> -1) Then
                    ctrl.Src = Page.ResolveUrl(ctrl.Src)
                End If
            End If

            ' UrlRewriting Issue - ResolveClientUrl gets called instead of ResolveUrl
            ' Manual Override to ResolveUrl
            If TypeOf c Is System.Web.UI.WebControls.HyperLink Then
                Dim ctrl As System.Web.UI.WebControls.HyperLink
                ctrl = DirectCast(c, System.Web.UI.WebControls.HyperLink)
                If (ctrl.NavigateUrl.IndexOf("~") <> -1) Then
                    ctrl.NavigateUrl = Page.ResolveUrl(ctrl.NavigateUrl)
                End If
                If (ctrl.ImageUrl.IndexOf("~") <> -1) Then
                    ctrl.ImageUrl = Page.ResolveUrl(ctrl.ImageUrl)
                End If
            End If

            ' Process child controls
            If includeChildren = True And c.HasControls() Then
                If TypeOf c Is PortalModuleBase Then
                    'Get Resource File Root from Controls LocalResourceFile Property
                    Dim ctrl As Entities.Modules.PortalModuleBase
                    ctrl = DirectCast(c, PortalModuleBase)
                    IterateControls(c.Controls, affectedControls, ctrl.LocalResourceFile)
                Else
                    Dim pi As PropertyInfo = c.GetType.GetProperty("LocalResourceFile")
                    If Not pi Is Nothing AndAlso Not pi.GetValue(c, Nothing) Is Nothing Then
                        'If controls has a LocalResourceFile property use this
                        IterateControls(c.Controls, affectedControls, pi.GetValue(c, Nothing).ToString())
                    Else
                        'Pass Resource File Root through
                        IterateControls(c.Controls, affectedControls, ResourceFileRoot)
                    End If
                End If
            End If

        End Sub    'ProcessControl

        ''' <summary>
        ''' <para>GetControlAttribute looks a the type of control and does it's best to find an AttributeCollection.</para>
        ''' </summary>
        ''' <param name="c">Control to find the AttributeCollection on</param>
        ''' <param name="affectedControls">ArrayList that hold the controls that have been localized. This is later used for the removal of the key attribute.</param>				
        ''' <returns>A string containing the key for the specified control or null if a key attribute wasn't found</returns>
        Friend Shared Function GetControlAttribute(ByVal c As Control, ByVal affectedControls As ArrayList) As String
            Dim ac As System.Web.UI.AttributeCollection = Nothing
            Dim key As String = Nothing

            If TypeOf c Is LiteralControl Then    ' LiteralControls don't have an attribute collection
                key = Nothing
                ac = Nothing
            Else
                If TypeOf c Is WebControl Then
                    Dim w As WebControl = DirectCast(c, WebControl)
                    ac = w.Attributes
                    key = ac(Services.Localization.Localization.KeyName)
                Else
                    If TypeOf c Is HtmlControl Then
                        Dim h As HtmlControl = DirectCast(c, HtmlControl)
                        ac = h.Attributes
                        key = ac(Services.Localization.Localization.KeyName)
                    Else
                        If TypeOf c Is UserControl Then
                            Dim u As UserControl = DirectCast(c, UserControl)
                            ac = u.Attributes
                            key = ac(Services.Localization.Localization.KeyName)
                            ' Use reflection to check for attribute key. This is a last ditch option
                        Else
                            Dim controlType As Type = c.GetType()
                            Dim attributeProperty As PropertyInfo = controlType.GetProperty("Attributes", GetType(System.Web.UI.AttributeCollection))
                            If Not attributeProperty Is Nothing Then
                                ac = DirectCast(attributeProperty.GetValue(c, Nothing), System.Web.UI.AttributeCollection)
                                key = ac(Services.Localization.Localization.KeyName)
                            End If
                        End If
                    End If
                End If    ' If the key was found add this AttributeCollection to the list that should have the key removed during Render
            End If
            If Not key Is Nothing And Not affectedControls Is Nothing Then
                affectedControls.Add(ac)
            End If
            Return key
        End Function    'GetControlAttribute

        ''' <summary>
        ''' <para>RemoveKeyAttribute remove the key attribute from the control. If this isn't done, then the HTML output will have 
        ''' a bad attribute on it which could cause some older browsers problems.</para>
        ''' </summary>
        ''' <param name="affectedControls">ArrayList that hold the controls that have been localized. This is later used for the removal of the key attribute.</param>		
        Public Shared Sub RemoveKeyAttribute(ByVal affectedControls As ArrayList)
            If affectedControls Is Nothing Then
                Return
            End If
            Dim i As Integer
            For i = 0 To affectedControls.Count - 1
                Dim ac As System.Web.UI.AttributeCollection = DirectCast(affectedControls(i), System.Web.UI.AttributeCollection)
                ac.Remove(Services.Localization.Localization.KeyName)
            Next i
        End Sub    'RemoveKeyAttribute

        Public Sub SetLanguage(ByVal value As String)
            ' save the pageculture as a cookie
            Dim cookie As System.Web.HttpCookie = Response.Cookies.Get("language")
            If (cookie Is Nothing) Then
                If value <> "" Then
                    cookie = New System.Web.HttpCookie("language", value)
                    Response.Cookies.Add(cookie)
                End If
            Else
                cookie.Value = value
                Response.Cookies.Set(cookie)
                If value = "" Then
                    Response.Cookies.Remove("language")
                End If
            End If
        End Sub

#End Region

#Region "Private Helpers"

        Private Sub Page_Error(ByVal Source As Object, ByVal e As System.EventArgs) Handles MyBase.Error
            Dim exc As Exception = Server.GetLastError
            Dim strURL As String = ApplicationURL()
            If Not Request.QueryString("error") Is Nothing Then
                strURL += IIf(strURL.IndexOf("?") = -1, "?", "&").ToString & "error=terminate"
            Else
                strURL += IIf(strURL.IndexOf("?") = -1, "?", "&").ToString & "error=" & Server.UrlEncode(exc.Message)
                If Not IsAdminControl() Then
                    strURL += "&content=0"
                End If
            End If
            ProcessPageLoadException(exc, strURL)
        End Sub

        ''' <summary>
        ''' <para>Render is called to render all the server controls. Just prior to rendering, the localization is performed.</para>
        ''' <seealso cref="System.Web.UI.Control.Render" />
        ''' </summary>
        ''' <param name="evt">An EventArgs that controls the event data.</param>
        Protected Overrides Sub OnPreRender(ByVal evt As EventArgs)
            Trace.Write("LocalizedPage::PreRender", String.Format("Performing substitutions for {0}", Thread.CurrentThread.CurrentUICulture.Name))

            MyBase.OnPreRender(evt)
            IterateControls(Controls, _localizedControls, LocalResourceFile)
        End Sub    'OnPreRender

        '
        ' This method overrides the Render() method for the page and moves the ViewState
        ' from its default location at the top of the page to the bottom of the page. This
        ' results in better search engine spidering. 
        '
        Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)

            Dim stringWriter As System.IO.StringWriter = New System.IO.StringWriter
            Dim htmlWriter As HtmlTextWriter = New HtmlTextWriter(stringWriter)

            Trace.Write("LocalizedPage::Render", "Removing resourcekey attribute from controls")
            RemoveKeyAttribute(_localizedControls)

            MyBase.Render(htmlWriter)

            Dim html As String = stringWriter.ToString()
            Dim StartPoint As Integer = html.IndexOf("<input type=""hidden"" name=""__VIEWSTATE""")
            If StartPoint >= 0 Then    'does __VIEWSTATE exist?
                Dim EndPoint As Integer = html.IndexOf("/>", StartPoint) + 2
                Dim ViewStateInput As String = html.Substring(StartPoint, EndPoint - StartPoint)
                html = html.Remove(StartPoint, EndPoint - StartPoint)
                Dim FormEndStart As Integer = html.IndexOf("</form>") - 1
                If FormEndStart >= 0 Then
                    html = html.Insert(FormEndStart, ViewStateInput)
                End If
            End If
            writer.Write(html)

        End Sub

        ''' <summary>
        ''' <para>IterateControls performs the high level localization for each control on the page.</para>
        ''' </summary>
        Private Overloads Sub IterateControls(ByVal affectedControls As ArrayList)
            IterateControls(Controls, affectedControls, Nothing)
        End Sub    'IterateControls

        Private Overloads Sub IterateControls(ByVal controls As ControlCollection, ByVal affectedControls As ArrayList, ByVal ResourceFileRoot As String)
            For Each c As Control In controls
                If TypeOf c Is Solpart.WebControls.SolpartMenu Then
                    'IterateMenuControls(DirectCast(c, Solpart.WebControls.SolpartMenu).MenuItems)
                Else
                    ProcessControl(c, affectedControls, True, ResourceFileRoot)
                End If
            Next
        End Sub    'IterateControls

#End Region

    End Class 'PageBase

End Namespace
