'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.Web
Imports System.Text.RegularExpressions

Imports DotNetNuke.Common
Imports DotNetNuke.Common.Utilities
Imports DotnetNuke.Entities.Portals
Imports DotNetNuke.Services.Url.FriendlyUrl
Imports DotNetNuke.Framework.Providers
Imports DotNetNuke.Entities.Tabs

Namespace DotNetNuke.Services.Url.FriendlyUrl

    Public Class DNNFriendlyUrlProvider

        Inherits FriendlyUrlProvider

        Private Const ProviderType As String = "friendlyUrl"
        Private _providerConfiguration As ProviderConfiguration = ProviderConfiguration.GetProviderConfiguration(ProviderType)
        Private _includePageName As Boolean

        Public Sub New()

            ' Read the configuration specific information for this provider
            Dim objProvider As Provider = CType(_providerConfiguration.Providers(_providerConfiguration.DefaultProvider), Provider)

            ' Read the attributes for this provider

            If Convert.ToString(objProvider.Attributes("includePageName")) <> "" Then
                _includePageName = Boolean.Parse(objProvider.Attributes("includePageName"))
            Else
                _includePageName = True
            End If

        End Sub

        Public ReadOnly Property IncludePageName() As Boolean
            Get
                Return _includePageName
            End Get
        End Property

#Region "Public Methods"

        Public Overloads Overrides Function FriendlyUrl(ByVal tab As TabInfo, ByVal path As String) As String

            Dim _portalSettings As PortalSettings = PortalController.GetCurrentPortalSettings

            Return FriendlyUrl(tab, path, glbDefaultPage, _portalSettings)

        End Function

        Public Overloads Overrides Function FriendlyUrl(ByVal tab As TabInfo, ByVal path As String, ByVal pageName As String) As String

            Dim _portalSettings As PortalSettings = PortalController.GetCurrentPortalSettings

            Return FriendlyUrl(tab, path, pageName, _portalSettings)

        End Function

        Public Overloads Overrides Function FriendlyUrl(ByVal tab As TabInfo, ByVal path As String, ByVal pageName As String, ByVal settings As PortalSettings) As String

            Dim friendlyPath As String = path
            Dim matchString As String = ""

            'Call GetFriendlyAlias to get the Alias part of the url
            friendlyPath = GetFriendlyAlias(path, settings.PortalAlias.HTTPAlias)

            'Call GetFriendlyQueryString to get the QueryString part of the url
            friendlyPath = GetFriendlyQueryString(tab, friendlyPath, settings, pageName)

            Return friendlyPath

        End Function

        Public Overloads Overrides Function FriendlyUrl(ByVal tab As TabInfo, ByVal path As String, ByVal pageName As String, ByVal portalAlias As String) As String

            Dim friendlyPath As String = path
            Dim matchString As String = ""

            'Call GetFriendlyAlias to get the Alias part of the url
            friendlyPath = GetFriendlyAlias(path, portalAlias)

            'Call GetFriendlyQueryString to get the QueryString part of the url
            friendlyPath = GetFriendlyQueryString(tab, friendlyPath, pageName)

            Return friendlyPath

        End Function
#End Region

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' AddPage adds the page to the friendly url
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="path">The path to format.</param>
        ''' <param name="pageName">The page name.</param>
        ''' <returns>The formatted url</returns>
        ''' <history>
        '''		[cnurse]	12/16/2004	created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function AddPage(ByVal path As String, ByVal pageName As String) As String

            Dim friendlyPath As String = path

            If (friendlyPath.EndsWith("/")) Then
                friendlyPath = friendlyPath & pageName
            Else
                friendlyPath = friendlyPath & "/" & pageName
            End If

            Return friendlyPath

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetFriendlyAlias gets the Alias root of the friendly url
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="path">The path to format.</param>
        ''' <param name="portalAlias">The portal alias of the site.</param>
        ''' <returns>The formatted url</returns>
        ''' <history>
        '''		[cnurse]	12/16/2004	created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function GetFriendlyAlias(ByVal path As String, ByVal portalAlias As String) As String

            Dim friendlyPath As String = path
            Dim matchString As String = ""

            If Not (portalAlias = Null.NullString) Then
                If Not (HttpContext.Current.Items("UrlRewrite:OriginalUrl") Is Nothing) Then
                    Dim originalUrl As String = HttpContext.Current.Items("UrlRewrite:OriginalUrl").ToString()

                    'For Each entry As String In arrAlias
                    Dim portalMatch As Match = Regex.Match(originalUrl, "^" & AddHTTP(portalAlias), RegexOptions.IgnoreCase)
                    If Not (portalMatch Is Match.Empty) Then
                        matchString = AddHTTP(portalAlias)
                    End If

                    If (matchString = "") Then
                        'Manage the special case where original url contains the alias as
                        'http://www.domain.com/Default.aspx?alias=www.domain.com/child"
                        portalMatch = Regex.Match(originalUrl, "^?alias=" & portalAlias, RegexOptions.IgnoreCase)
                        If Not (portalMatch Is Match.Empty) Then
                            matchString = AddHTTP(portalAlias)
                        End If
                    End If
                End If
            End If

            If (matchString <> "") Then
                If (path.IndexOf("~") <> -1) Then
                    friendlyPath = friendlyPath.Replace("~", matchString)
                Else
                    friendlyPath = matchString & friendlyPath
                End If
            Else
                friendlyPath = ResolveUrl(friendlyPath)
            End If

            Return friendlyPath

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetFriendlyQueryString gets the Querystring part of the friendly url
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="path">The path to format.</param>
        ''' <param name="settings">The Portal Settings for the site.</param>
        ''' <returns>The formatted url</returns>
        ''' <history>
        '''		[cnurse]	12/16/2004	created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function GetFriendlyQueryString(ByVal tab As TabInfo, ByVal path As String, ByVal settings As PortalSettings, ByVal pageName As String) As String

            Dim friendlyPath As String = path
            Dim queryStringMatch As Match = Regex.Match(friendlyPath, "(.[^\\?]*)\\?(.*)", RegexOptions.IgnoreCase)
            Dim queryStringSpecialChars As String = ""

            If Not (queryStringMatch Is Match.Empty) Then
                friendlyPath = queryStringMatch.Groups(1).Value
                friendlyPath = Regex.Replace(friendlyPath, glbDefaultPage, "", RegexOptions.IgnoreCase)

                Dim queryString As String = queryStringMatch.Groups(2).Value.Replace("&amp;", "&")
                If (queryString.StartsWith("?")) Then
                    queryString = queryString.TrimStart(Convert.ToChar("?"))
                End If

                Dim nameValuePairs As String() = queryString.Split(Convert.ToChar("&"))
                For i As Integer = 0 To nameValuePairs.Length - 1
                    Dim pathToAppend As String = ""
                    Dim pair As String() = nameValuePairs(i).Split(Convert.ToChar("="))

                    If (friendlyPath.EndsWith("/")) Then
                        pathToAppend = pathToAppend & pair(0)
                    Else
                        pathToAppend = pathToAppend & "/" & pair(0)
                    End If

                    If (pair.Length > 1) Then
                        If (pair(1).Length > 0) Then

                            If (Regex.IsMatch(pair(1), "[^a-zA-Z0-9 ]") = False) Then

                                ' Contains Non-AlphaNumeric Characters

                                If (pair(0).ToLower() = "tabid") Then
                                    If (IsNumeric(pair(1))) Then
                                        If Not (tab Is Nothing) Then
                                            Dim tabId As Integer = Convert.ToInt32(pair(1))
                                            If (tab.TabID = tabId) Then
                                                If (tab.TabPath <> Null.NullString) And IncludePageName Then
                                                    pathToAppend = tab.TabPath.Replace("//", "/").TrimStart("/"c) & "/" & pathToAppend
                                                End If
                                            End If
                                        End If
                                    End If
                                End If

                                pathToAppend = pathToAppend & "/" & pair(1).Replace(" ", System.Web.HttpUtility.UrlEncode((Chr(32)).ToString()))

                            Else

                                ' Rewrite into URL, contains only alphanumeric and the % or space
                                If (queryStringSpecialChars.Length = 0) Then
                                    queryStringSpecialChars = pair(0) & "=" & pair(1)
                                Else
                                    queryStringSpecialChars = queryStringSpecialChars & "&" & pair(0) & "=" & pair(1)
                                End If

                                pathToAppend = ""

                            End If

                        Else
                            pathToAppend = pathToAppend & "/" & System.Web.HttpUtility.UrlEncode((Chr(32)).ToString())
                        End If
                    End If

                    friendlyPath = friendlyPath & pathToAppend
                Next
            End If

            If (queryStringSpecialChars.Length > 0) Then
                Return AddPage(friendlyPath, pageName) & "?" & queryStringSpecialChars
            Else
                Return AddPage(friendlyPath, pageName)
            End If

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetFriendlyQueryString gets the Querystring part of the friendly url
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="tab">The tab whose url is being formatted.</param>
        ''' <param name="path">The path to format.</param>
        ''' <returns>The formatted url</returns>
        ''' <history>
        '''		[cnurse]	12/16/2004	created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function GetFriendlyQueryString(ByVal tab As TabInfo, ByVal path As String, ByVal pageName As String) As String

            Dim friendlyPath As String = path
            Dim queryStringMatch As Match = Regex.Match(friendlyPath, "(.[^\\?]*)\\?(.*)", RegexOptions.IgnoreCase)
            Dim queryStringSpecialChars As String = ""

            If Not (queryStringMatch Is Match.Empty) Then
                friendlyPath = queryStringMatch.Groups(1).Value
                friendlyPath = Regex.Replace(friendlyPath, glbDefaultPage, "", RegexOptions.IgnoreCase)

                Dim queryString As String = queryStringMatch.Groups(2).Value.Replace("&amp;", "&")
                If (queryString.StartsWith("?")) Then
                    queryString = queryString.TrimStart(Convert.ToChar("?"))
                End If

                Dim nameValuePairs As String() = queryString.Split(Convert.ToChar("&"))
                For i As Integer = 0 To nameValuePairs.Length - 1
                    Dim pathToAppend As String = ""
                    Dim pair As String() = nameValuePairs(i).Split(Convert.ToChar("="))

                    'Add name part of name/value pair
                    If (friendlyPath.EndsWith("/")) Then
                        pathToAppend = pathToAppend & pair(0)
                    Else
                        pathToAppend = pathToAppend & "/" & pair(0)
                    End If

                    If (pair.Length > 1) Then
                        If (pair(1).Length > 0) Then

                            If (Regex.IsMatch(pair(1), "[^a-zA-Z0-9 ]") = False) Then

                                ' Contains Non-AlphaNumeric Characters
                                If (pair(0).ToLower() = "tabid") Then
                                    If (IsNumeric(pair(1))) Then
                                        If Not (tab Is Nothing) Then
                                            Dim tabId As Integer = Convert.ToInt32(pair(1))
                                            If (tab.TabID = tabId) Then
                                                If (tab.TabPath <> Null.NullString) And IncludePageName Then
                                                    pathToAppend = tab.TabPath.Replace("//", "/").TrimStart("/"c) & "/" & pathToAppend
                                                End If
                                            End If
                                        End If
                                    End If
                                End If

                                pathToAppend = pathToAppend & "/" & pair(1).Replace(" ", System.Web.HttpUtility.UrlEncode((Chr(32)).ToString()))

                            Else

                                ' Rewrite into URL, contains only alphanumeric and the % or space
                                If (queryStringSpecialChars.Length = 0) Then
                                    queryStringSpecialChars = pair(0) & "=" & pair(1)
                                Else
                                    queryStringSpecialChars = queryStringSpecialChars & "&" & pair(0) & "=" & pair(1)
                                End If

                                pathToAppend = ""

                            End If
                        Else
                            pathToAppend = pathToAppend & "/" & System.Web.HttpUtility.UrlEncode((Chr(32)).ToString())
                        End If
                    End If

                    friendlyPath = friendlyPath & pathToAppend
                Next
            End If

            If (queryStringSpecialChars.Length > 0) Then
                Return AddPage(friendlyPath, pageName) & "?" & queryStringSpecialChars
            Else
                Return AddPage(friendlyPath, pageName)
            End If
        End Function

    End Class

End Namespace

