'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Web.Caching
Imports System.Reflection
Imports DotNetNuke.Services.Log.EventLog
Imports DotNetNuke.Services.Log.EventLog.DBLoggingProvider

Namespace DotNetNuke.Services.Log.EventLog.DBLoggingProvider.Data

    Public MustInherit Class DataProvider

#Region "Shared/Static Methods"
        ' provider constants - eliminates need for Reflection later
        Private Const ProviderType As String = "data"    ' maps to <sectionGroup> in web.config
        Private Const ProviderNamespace As String = "DotNetNuke.Services.Log.EventLog.DBLoggingProvider.Data"    ' project namespace
        Private Const ProviderAssemblyName As String = "DotNetNuke.Provider.DBLoggingProvider"    ' project assemblyname

        Public Shared Shadows Function Instance() As DataProvider

            Dim strCacheKey As String = ProviderNamespace & "." & ProviderType & "provider"

            ' Use the cache because the reflection used later is expensive
            Dim objConstructor As ConstructorInfo = CType(Common.Utilities.DataCache.GetCache(strCacheKey), ConstructorInfo)

            If objConstructor Is Nothing Then
                ' Get the provider configuration based on the type
                Dim objProviderConfiguration As Framework.Providers.ProviderConfiguration = Framework.Providers.ProviderConfiguration.GetProviderConfiguration(ProviderType)

                ' The assembly should be in \bin or GAC, so we simply need to get an instance of the type
                Try

                    ' Override the typename if a ProviderName is specified ( this allows the application to load a different DataProvider assembly for custom modules )
                    Dim strTypeName As String = ProviderNamespace & "." & objProviderConfiguration.DefaultProvider & ", " & ProviderAssemblyName & "." & objProviderConfiguration.DefaultProvider


                    ' Use reflection to store the constructor of the class that implements DataProvider
                    Dim t As Type = Type.GetType(strTypeName, True)
                    objConstructor = t.GetConstructor(System.Type.EmptyTypes)

                    ' Insert the type into the cache
                    Common.Utilities.DataCache.SetCache(strCacheKey, objConstructor)

                Catch e As Exception
                    ' Could not load the provider - this is likely due to binary compatibility issues 
                    ProcessSchedulerException(e)
                End Try
            End If

            Return CType(objConstructor.Invoke(Nothing), DataProvider)

        End Function

#End Region

#Region "Abstract Methods"
        '---------------------------------------------------------------------
        ' TODO Declare DAL methods. Should be implemented in each DAL DataProvider
        ' Use CodeSmith templates to generate this code
        '---------------------------------------------------------------------


        Public MustOverride Sub AddLog(ByVal LogGUID As String, ByVal LogTypeKey As String, ByVal LogUserID As Integer, ByVal LogUserName As String, ByVal LogPortalID As Integer, ByVal LogPortalName As String, ByVal LogCreateDate As Date, ByVal LogServerName As String, ByVal LogProperties As String, ByVal LogConfigID As Integer)
        Public MustOverride Sub DeleteLog(ByVal LogGUID As String)
        Public MustOverride Sub PurgeLog()
        Public MustOverride Sub ClearLog()

        Public MustOverride Sub AddLogTypeConfigInfo(ByVal LoggingIsActive As Boolean, ByVal LogTypeKey As String, ByVal LogTypePortalID As String, ByVal KeepMostRecent As Integer, ByVal EmailNotificationIsActive As Boolean, ByVal Threshold As Integer, ByVal NotificationThresholdTime As Integer, ByVal NotificationThresholdTimeType As Integer, ByVal MailFromAddress As String, ByVal MailToAddress As String)
        Public MustOverride Sub DeleteLogTypeConfigInfo(ByVal ID As String)
        Public MustOverride Function GetLogTypeConfigInfo() As IDataReader
        Public MustOverride Function GetLogTypeConfigInfoByID(ByVal ID As Integer) As IDataReader
        Public MustOverride Sub UpdateLogTypeConfigInfo(ByVal ID As String, ByVal LoggingIsActive As Boolean, ByVal LogTypeKey As String, ByVal LogTypePortalID As String, ByVal KeepMostRecent As Integer, ByVal LogFileName As String, ByVal EmailNotificationIsActive As Boolean, ByVal Threshold As Integer, ByVal NotificationThresholdTime As Integer, ByVal NotificationThresholdTimeType As Integer, ByVal MailFromAddress As String, ByVal MailToAddress As String)

        Public MustOverride Sub AddLogType(ByVal LogTypeKey As String, ByVal LogTypeFriendlyName As String, ByVal LogTypeDescription As String, ByVal LogTypeCSSClass As String, ByVal LogTypeOwner As String)
        Public MustOverride Sub UpdateLogType(ByVal LogTypeKey As String, ByVal LogTypeFriendlyName As String, ByVal LogTypeDescription As String, ByVal LogTypeCSSClass As String, ByVal LogTypeOwner As String)
        Public MustOverride Sub DeleteLogType(ByVal LogTypeKey As String)
        Public MustOverride Function GetLogTypeInfo() As IDataReader

        Public MustOverride Function GetLog() As IDataReader
        Public MustOverride Function GetLog(ByVal PortalID As Integer) As IDataReader
        Public MustOverride Function GetLog(ByVal PortalID As Integer, ByVal LogType As String) As IDataReader
        Public MustOverride Function GetLog(ByVal LogType As String) As IDataReader

        Public MustOverride Function GetLog(ByVal PageSize As Integer, ByVal PageIndex As Integer) As IDataReader
        Public MustOverride Function GetLog(ByVal PortalID As Integer, ByVal PageSize As Integer, ByVal PageIndex As Integer) As IDataReader
        Public MustOverride Function GetLog(ByVal PortalID As Integer, ByVal LogType As String, ByVal PageSize As Integer, ByVal PageIndex As Integer) As IDataReader
        Public MustOverride Function GetLog(ByVal LogType As String, ByVal PageSize As Integer, ByVal PageIndex As Integer) As IDataReader

        Public MustOverride Function GetSingleLog(ByVal LogGUID As String) As IDataReader
        Public MustOverride Function GetEventLogPendingNotifConfig() As IDataReader
        Public MustOverride Function GetEventLogPendingNotif(ByVal LogConfigID As Integer) As IDataReader
        Public MustOverride Sub UpdateEventLogPendingNotif(ByVal LogConfigID As Integer)

#End Region

    End Class

End Namespace
