'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports DotNetNuke.Security.Roles
Imports DotNetNuke.UI.Utilities

Namespace DotNetNuke.Modules.Admin.Security

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The EditRoles PortalModuleBase is used to manage a Security Role
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
    '''                       and localisation
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class EditRoles

        Inherits DotNetNuke.Entities.Modules.PortalModuleBase

#Region "Controls"

        Protected WithEvents pnlSettings As System.Web.UI.WebControls.Panel

        'Basic Settings Section
        Protected dshBasic As UI.UserControls.SectionHeadControl
        Protected tblBasic As System.Web.UI.HtmlControls.HtmlTable
        Protected lblBasicSettingsHelp As System.Web.UI.WebControls.Label
        Protected WithEvents txtRoleName As System.Web.UI.WebControls.TextBox
        Protected WithEvents valRoleName As System.Web.UI.WebControls.RequiredFieldValidator
        Protected plRoleName As UI.UserControls.LabelControl
        Protected WithEvents txtDescription As System.Web.UI.WebControls.TextBox
        Protected plDescription As UI.UserControls.LabelControl
        Protected WithEvents chkIsPublic As System.Web.UI.WebControls.CheckBox
        Protected plIsPublic As UI.UserControls.LabelControl
        Protected WithEvents chkAutoAssignment As System.Web.UI.WebControls.CheckBox
        Protected plAutoAssignment As UI.UserControls.LabelControl
        Protected WithEvents lblRoleName As System.Web.UI.WebControls.Label

        'Advanced Settings Section
        Protected dshAdvanced As UI.UserControls.SectionHeadControl
        Protected tblAdvanced As System.Web.UI.HtmlControls.HtmlTable
        Protected lblAdvancedSettingsHelp As System.Web.UI.WebControls.Label
        Protected WithEvents txtServiceFee As System.Web.UI.WebControls.TextBox
        Protected plServiceFee As UI.UserControls.LabelControl
        Protected WithEvents txtBillingPeriod As System.Web.UI.WebControls.TextBox
        Protected plBillingPeriod As UI.UserControls.LabelControl
        Protected WithEvents cboBillingFrequency As System.Web.UI.WebControls.DropDownList
        Protected WithEvents txtTrialFee As System.Web.UI.WebControls.TextBox
        Protected plTrialFee As UI.UserControls.LabelControl
        Protected WithEvents txtTrialPeriod As System.Web.UI.WebControls.TextBox
        Protected plTrialPeriod As UI.UserControls.LabelControl
        Protected WithEvents cboTrialFrequency As System.Web.UI.WebControls.DropDownList
        Protected WithEvents valServiceFee1 As System.Web.UI.WebControls.CompareValidator
        Protected WithEvents valServiceFee2 As System.Web.UI.WebControls.CompareValidator
        Protected WithEvents valBillingPeriod1 As System.Web.UI.WebControls.CompareValidator
        Protected WithEvents valBillingPeriod2 As System.Web.UI.WebControls.CompareValidator
        Protected WithEvents valTrialFee1 As System.Web.UI.WebControls.CompareValidator
        Protected WithEvents valTrialFee2 As System.Web.UI.WebControls.CompareValidator
        Protected WithEvents valTrialPeriod1 As System.Web.UI.WebControls.CompareValidator
        Protected WithEvents valTrialPeriod2 As System.Web.UI.WebControls.CompareValidator

        'Tasks
        Protected WithEvents cmdUpdate As System.Web.UI.WebControls.LinkButton
        Protected WithEvents cmdCancel As System.Web.UI.WebControls.LinkButton
        Protected WithEvents cmdDelete As System.Web.UI.WebControls.LinkButton
        Protected WithEvents cmdManage As System.Web.UI.WebControls.LinkButton

        'Not implemented
        Protected WithEvents txtEvery As System.Web.UI.WebControls.TextBox
        Protected WithEvents cboPeriod As System.Web.UI.WebControls.DropDownList
        Protected WithEvents txtStartDate As System.Web.UI.WebControls.TextBox
        Protected WithEvents pnlNotImplemented As System.Web.UI.WebControls.Panel

#End Region

#Region "Private Members"

        Private RoleID As Integer = -1

#End Region

#Region "Private Methods"
        Private Sub ActivateControls(ByVal enabled As Boolean)
            txtDescription.Enabled = enabled
            chkIsPublic.Enabled = enabled
            chkAutoAssignment.Enabled = enabled
            txtServiceFee.Enabled = enabled
            txtBillingPeriod.Enabled = enabled
            cboBillingFrequency.Enabled = enabled
            txtTrialFee.Enabled = enabled
            txtTrialPeriod.Enabled = enabled
            cboTrialFrequency.Enabled = enabled
        End Sub
#End Region

#Region "Event Handlers"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Page_Load runs when the control is loaded
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
        '''                       and localisation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
            Try
                If Not (Request.QueryString("RoleID") Is Nothing) Then
                    RoleID = Int32.Parse(Request.QueryString("RoleID"))
                End If

				If Page.IsPostBack = False Then
					ClientAPI.AddButtonConfirm(cmdDelete, Services.Localization.Localization.GetString("DeleteItem"))

					Dim objUser As New RoleController

					Dim ctlList As New Common.Lists.ListController
					Dim colFrequencies As Common.Lists.ListEntryInfoCollection = ctlList.GetListEntryInfoCollection("Frequency", "")

					cboBillingFrequency.DataSource = colFrequencies
					cboBillingFrequency.DataBind()
					cboBillingFrequency.Items.FindByValue("N").Selected = True

					cboTrialFrequency.DataSource = colFrequencies
					cboTrialFrequency.DataBind()
					cboTrialFrequency.Items.FindByValue("N").Selected = True

					If RoleID <> -1 Then
						lblRoleName.Visible = True
						txtRoleName.Visible = False
						valRoleName.Enabled = False

						Dim objRoleInfo As RoleInfo = objUser.GetRole(RoleID, PortalSettings.PortalId)

						If Not objRoleInfo Is Nothing Then
							lblRoleName.Text = objRoleInfo.RoleName
							txtDescription.Text = objRoleInfo.Description
							If Format(objRoleInfo.ServiceFee, "#,##0.00") <> "0.00" Then
								txtServiceFee.Text = Format(objRoleInfo.ServiceFee, "#,##0.00")
								txtBillingPeriod.Text = objRoleInfo.BillingPeriod.ToString
								If Not cboBillingFrequency.Items.FindByValue(objRoleInfo.BillingFrequency) Is Nothing Then
									cboBillingFrequency.ClearSelection()
									cboBillingFrequency.Items.FindByValue(objRoleInfo.BillingFrequency).Selected = True
								End If
							End If
							If objRoleInfo.TrialFrequency <> "N" Then
								txtTrialFee.Text = Format(objRoleInfo.TrialFee, "#,##0.00")
								txtTrialPeriod.Text = objRoleInfo.TrialPeriod.ToString
								If Not cboTrialFrequency.Items.FindByValue(objRoleInfo.TrialFrequency) Is Nothing Then
									cboTrialFrequency.ClearSelection()
									cboTrialFrequency.Items.FindByValue(objRoleInfo.TrialFrequency).Selected = True
								End If
							End If
							chkIsPublic.Checked = objRoleInfo.IsPublic
							chkAutoAssignment.Checked = objRoleInfo.AutoAssignment
						Else						 ' security violation attempt to access item not related to this Module
							Response.Redirect(NavigateURL("Security Roles"))
						End If

						If RoleID = PortalSettings.AdministratorRoleId Or RoleID = PortalSettings.RegisteredRoleId Then
							cmdDelete.Visible = False
							cmdUpdate.Visible = False
							ActivateControls(False)
						End If

						If RoleID = PortalSettings.RegisteredRoleId Then
							cmdManage.Visible = False
						End If
					Else
						cmdDelete.Visible = False
						cmdManage.Visible = False
						lblRoleName.Visible = False
						txtRoleName.Visible = True
					End If

				End If

			Catch exc As Exception			 'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdUpdate_Click runs when the update Button is clicked
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
        '''                       and localisation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdUpdate_Click(ByVal sender As Object, ByVal e As EventArgs) Handles cmdUpdate.Click
            Try
                If Page.IsValid Then

                    Dim sglServiceFee As Single = 0
                    Dim intBillingPeriod As Integer = 1
                    Dim strBillingFrequency As String = "N"

                    If txtServiceFee.Text <> "" And txtBillingPeriod.Text <> "" And cboBillingFrequency.SelectedItem.Value <> "N" Then
                        sglServiceFee = Single.Parse(txtServiceFee.Text)
                        intBillingPeriod = Integer.Parse(txtBillingPeriod.Text)
                        strBillingFrequency = cboBillingFrequency.SelectedItem.Value
                    End If

                    Dim sglTrialFee As Single = 0
                    Dim intTrialPeriod As Integer = 1
                    Dim strTrialFrequency As String = "N"

                    If sglServiceFee <> 0 And txtTrialFee.Text <> "" And txtTrialPeriod.Text <> "" And cboTrialFrequency.SelectedItem.Value <> "N" Then
                        sglTrialFee = Single.Parse(txtTrialFee.Text)
                        intTrialPeriod = Integer.Parse(txtTrialPeriod.Text)
                        strTrialFrequency = cboTrialFrequency.SelectedItem.Value
                    End If

                    Dim objRoleController As New RoleController
                    Dim objRoleInfo As New RoleInfo
                    objRoleInfo.PortalID = PortalId
                    objRoleInfo.RoleID = RoleID
                    objRoleInfo.RoleName = txtRoleName.Text
                    objRoleInfo.Description = txtDescription.Text
                    objRoleInfo.ServiceFee = sglServiceFee
                    objRoleInfo.BillingPeriod = intBillingPeriod
                    objRoleInfo.BillingFrequency = strBillingFrequency
                    objRoleInfo.TrialFee = sglTrialFee
                    objRoleInfo.TrialPeriod = intTrialPeriod
                    objRoleInfo.TrialFrequency = strTrialFrequency
                    objRoleInfo.IsPublic = chkIsPublic.Checked
                    objRoleInfo.AutoAssignment = chkAutoAssignment.Checked

                    Dim objEventLog As New Services.Log.EventLog.EventLogController
                    If RoleID = -1 Then
                        objRoleController.AddRole(objRoleInfo)
						objEventLog.AddLog(objRoleInfo, PortalSettings, UserId, "", Services.Log.EventLog.EventLogController.EventLogType.ROLE_CREATED)
                    Else
                        objRoleController.UpdateRole(objRoleInfo)
						objEventLog.AddLog(objRoleInfo, PortalSettings, UserId, "", Services.Log.EventLog.EventLogController.EventLogType.ROLE_UPDATED)
                    End If

                    'Clear Roles Cache
                    DataCache.RemoveCache("GetRoles")

                    Response.Redirect(NavigateURL())

                End If

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdDelete_Click runs when the delete Button is clicked
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
        '''                       and localisation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdDelete_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdDelete.Click
            Try
                Dim objUser As New RoleController

                objUser.DeleteRole(RoleID, PortalSettings.PortalId)
                Dim objEventLog As New Services.Log.EventLog.EventLogController
				objEventLog.AddLog("RoleID", RoleID.ToString, PortalSettings, UserId, Services.Log.EventLog.EventLogController.EventLogType.ROLE_DELETED)

                'Clear Roles Cache
                DataCache.RemoveCache("GetRoles")

                Response.Redirect(NavigateURL())

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdCancel_Click runs when the cancel Button is clicked
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
        '''                       and localisation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdCancel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
            Try
                Response.Redirect(NavigateURL())

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdManage_Click runs when the Manage Users Button is clicked
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
        '''                       and localisation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdManage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdManage.Click
            Try
                Response.Redirect(NavigateURL(Me.TabId, "User Roles", "RoleId=" & RoleID))
            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

#End Region

#Region " Web Form Designer Generated Code "

        'This call is required by the Web Form Designer.
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()

        End Sub

        Private Sub Page_Init(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Init
            'CODEGEN: This method call is required by the Web Form Designer
            'Do not modify it using the code editor.
            InitializeComponent()
        End Sub

#End Region

    End Class

End Namespace
