'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System.Threading
Imports System.Web.Mail
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Security.Roles
Imports DotNetNuke.Services.FileSystem

Namespace DotNetNuke.Modules.Admin.Users

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The BulkEmail PortalModuleBase is used to manage a Bulk Email mesages
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[cnurse]	9/13/2004	Updated to reflect design changes for Help, 508 support
    '''                       and localisation
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class BulkEmail
        Inherits DotNetNuke.Entities.Modules.PortalModuleBase

#Region "Controls"

        Protected WithEvents pnlSettings As System.Web.UI.WebControls.Panel

        'Basic Settings Section
        Protected dshBasic As UI.UserControls.SectionHeadControl
        Protected tblBasic As System.Web.UI.HtmlControls.HtmlTable
        Protected lblBasicSettingsHelp As System.Web.UI.WebControls.Label
        Protected WithEvents chkRoles As System.Web.UI.WebControls.CheckBoxList
        Protected plRoles As UI.UserControls.LabelControl
        Protected WithEvents txtEmail As System.Web.UI.WebControls.TextBox
        Protected plEmail As UI.UserControls.LabelControl
        Protected WithEvents txtSubject As System.Web.UI.WebControls.TextBox
        Protected plSubject As UI.UserControls.LabelControl

        'Message Section
        Protected dshMessage As UI.UserControls.SectionHeadControl
        Protected tblMessage As System.Web.UI.HtmlControls.HtmlTable
        Protected lblMessageHelp As System.Web.UI.WebControls.Label
        Protected WithEvents teMessage As UI.UserControls.TextEditor

        'Advanced Settings Section
        Protected dshAdvanced As UI.UserControls.SectionHeadControl
        Protected tblAdvanced As System.Web.UI.HtmlControls.HtmlTable
        Protected lblAdvancedSettingsHelp As System.Web.UI.WebControls.Label
        Protected ctlAttachment As UI.UserControls.UrlControl
        Protected plAttachment As UI.UserControls.LabelControl
        Protected WithEvents cboPriority As System.Web.UI.WebControls.DropDownList
        Protected plPriority As UI.UserControls.LabelControl
        Protected WithEvents cboSendMethod As System.Web.UI.WebControls.DropDownList
        Protected plSendMethod As UI.UserControls.LabelControl
        Protected WithEvents optSendAction As System.Web.UI.WebControls.RadioButtonList
        Protected plSendAction As UI.UserControls.LabelControl

        'tasks
        Protected WithEvents cmdSend As System.Web.UI.WebControls.LinkButton
#End Region

#Region "Event Handlers"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Page_Load runs when the control is loaded
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
        '''                       and localisation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
            Try

                If Not Page.IsPostBack Then
                    Dim objRoleController As New RoleController
                    chkRoles.DataSource = objRoleController.GetPortalRoles(PortalId)
                    chkRoles.DataBind()

                    'Dim FileList As ArrayList = GetFileList(PortalId)
                    'cboAttachment.DataSource = FileList
                    'cboAttachment.DataBind()
                    'cmdUpload.NavigateUrl = NavigateURL("File Manager")

                End If

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' cmdSend_Click runs when the cmdSend Button is clicked
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[cnurse]	9/10/2004	Updated to reflect design changes for Help, 508 support
        '''                       and localisation
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub cmdSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSend.Click
            Try
                Dim objRecipients As New ArrayList

                Dim objRole As ListItem
                Dim objListItem As ListItem
                Dim objUsers As New UserController
                Dim objUser As UserInfo
                Dim strUser As String

                ' load all user emails based on roles selected
                For Each objRole In chkRoles.Items
                    If objRole.Selected Then
                        For Each strUser In Microsoft.ScalableHosting.Security.Roles.GetUsersInRole(objRole.Value)
                            objUser = objUsers.GetUserByUsername(PortalId, strUser)
                            If objUser.Membership.Approved Then
                                objListItem = New ListItem(objUser.Membership.Email, objUser.FullName)
                                If Not objRecipients.Contains(objListItem) Then
                                    objRecipients.Add(objListItem)
                                End If
                            End If
                        Next
                    End If
                Next

                ' load emails specified in email distribution list
                If txtEmail.Text <> "" Then
                    Dim arrEmail As Array = Split(txtEmail.Text, ";")
                    Dim strEmail As String
                    For Each strEmail In arrEmail
                        objListItem = New ListItem(strEmail, strEmail)
                        If Not objRecipients.Contains(objListItem) Then
                            objRecipients.Add(objListItem)
                        End If
                    Next
                End If

                ' create object
                Dim objSendBulkEMail As New SendBulkEMail

                objSendBulkEMail.Recipients = objRecipients
                Select Case cboPriority.SelectedItem.Value
                    Case "1"
                        objSendBulkEMail.Priority = MailPriority.High
                    Case "2"
                        objSendBulkEMail.Priority = MailPriority.Normal
                    Case "3"
                        objSendBulkEMail.Priority = MailPriority.Low
                End Select
                objSendBulkEMail.Subject = txtSubject.Text
                If teMessage.Mode = "BASIC" Then
                    objSendBulkEMail.BodyFormat = MailFormat.Text
                Else
                    objSendBulkEMail.BodyFormat = MailFormat.Html
                    ' Add Base Href for any images inserted in to the email.
                    objSendBulkEMail.Body = "<Base Href='" & PortalSettings.PortalAlias.HTTPAlias & "'>"
                End If
                objSendBulkEMail.Body &= teMessage.Text
                If ctlAttachment.Url.StartsWith("FileID=") Then
                    Dim fileId As Integer = Integer.Parse(ctlAttachment.Url.Substring(7))
                    Dim objFileController As New FileController
                    Dim objFileInfo As FileInfo = objFileController.GetFileById(fileId, PortalId)

                    objSendBulkEMail.Attachment = PortalSettings.HomeDirectoryMapPath & objFileInfo.Folder & objFileInfo.FileName

                End If
                objSendBulkEMail.SendMethod = cboSendMethod.SelectedItem.Value
                objSendBulkEMail.SMTPServer = Convert.ToString(PortalSettings.HostSettings("SMTPServer"))
                objSendBulkEMail.SMTPAuthentication = Convert.ToString(PortalSettings.HostSettings("SMTPAuthentication"))
                objSendBulkEMail.SMTPUsername = Convert.ToString(PortalSettings.HostSettings("SMTPUsername"))
                objSendBulkEMail.SMTPPassword = Convert.ToString(PortalSettings.HostSettings("SMTPPassword"))
                objSendBulkEMail.Administrator = PortalSettings.Email
                objSendBulkEMail.Heading = Services.Localization.Localization.GetString("Heading", Me.LocalResourceFile)

                ' send mail
                If optSendAction.SelectedItem.Value = "S" Then
                    objSendBulkEMail.Send()
                Else    ' ansynchronous uses threading
                    Dim objThread As New Thread(AddressOf objSendBulkEMail.Send)
                    objThread.Start()
                End If

                ' completed
                UI.Skins.Skin.AddModuleMessage(Me, Services.Localization.Localization.GetString("MessageSent", Me.LocalResourceFile), DotNetNuke.UI.Skins.Controls.ModuleMessage.ModuleMessageType.GreenSuccess)

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

#End Region

#Region " Web Form Designer Generated Code "

        'This call is required by the Web Form Designer.
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()

        End Sub

        Private Sub Page_Init(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Init
            'CODEGEN: This method call is required by the Web Form Designer
            'Do not modify it using the code editor.
            InitializeComponent()
        End Sub

#End Region

    End Class

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The SendBulkEMail Class is a helper class tht manages the sending of Email
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[cnurse]	9/13/2004	Updated to reflect design changes for Help, 508 support
    '''                       and localisation
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class SendBulkEMail

        Inherits DotNetNuke.Entities.Modules.PortalModuleBase

        Public Recipients As ArrayList
        Public Priority As MailPriority
        Public Subject As String
        Public BodyFormat As MailFormat
        Public Body As String
        Public Attachment As String
        Public SendMethod As String
        Public SMTPServer As String
        Public SMTPAuthentication As String
        Public SMTPUsername As String
        Public SMTPPassword As String
        Public Administrator As String
        Public Heading As String

        Public Sub Send()
            Try

                Dim endDelimit As String
                'Use either vbCrLF or <br><br> depending on BodyFormat.
                If BodyFormat = MailFormat.Html Then
                    endDelimit = "<br>"
                Else
                    endDelimit = vbCrLf
                End If

                Dim strConfirmation As String
                strConfirmation = "Bulk Email Operation Started: " & Now().ToString & endDelimit & endDelimit

                ' send to recipients
                Dim strBody As String
                Dim intRecipients As Integer = 0
                Dim intMessages As Integer = 0
                Dim strDistributionList As String = ""
                Dim objRecipient As ListItem

                Select Case SendMethod
                    Case "TO"
                        For Each objRecipient In Recipients
                            If objRecipient.Text <> "" Then
                                intRecipients += 1
                                strBody = Heading & objRecipient.Value & "," & endDelimit & endDelimit & Body
                                SendNotification(Administrator, objRecipient.Text, "", "", Priority, Subject, BodyFormat, System.Text.Encoding.UTF8, strBody, Attachment, SMTPServer, SMTPAuthentication, SMTPUsername, SMTPPassword)
                                intMessages += 1
                            End If
                        Next
                    Case "BCC"
                        For Each objRecipient In Recipients
                            If objRecipient.Text <> "" Then
                                intRecipients += 1
                                strDistributionList += "; " & objRecipient.Text
                            End If
                        Next
                        intMessages = 1
                        strBody = Body
                        SendNotification(Administrator, "", "", strDistributionList, Priority, Subject, BodyFormat, System.Text.Encoding.UTF8, strBody, Attachment, SMTPServer, SMTPAuthentication, SMTPUsername, SMTPPassword)
                End Select

                ' send confirmation
                strConfirmation += "Email Recipients: " & intRecipients.ToString() & endDelimit
                strConfirmation += "Email Messages: " & intMessages.ToString() & endDelimit & endDelimit
                strConfirmation += "Bulk Email Operation Completed: " & Now().ToString & endDelimit
                SendNotification(Administrator, Administrator, "", "", Priority, "Bulk Email Confirmation", BodyFormat, System.Text.Encoding.UTF8, strConfirmation, "", SMTPServer, SMTPAuthentication, SMTPUsername, SMTPPassword)

            Catch exc As Exception
                ' send mail failure
                System.Diagnostics.Debug.Write(exc.Message)
            End Try

        End Sub

    End Class

End Namespace
