'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2005
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Collections
Imports System.Web.UI

Namespace DotNetNuke.UI.WebControls
	''' <summary>
	''' Represents a collection of <see cref="NodeImage">NodeImage</see> objects.
	''' </summary>
	Public Class NodeImageCollection
		Inherits CollectionBase
		Implements IStateManager

		Protected _isTrackingViewState As Boolean

		''' <summary>
		''' Initializes a new instance of the <see cref="NodeImageCollection">NodeImageCollection</see> class.
		''' </summary>
		Public Sub New()
			MyBase.New()
		End Sub

		''' <summary>
		''' Initializes a new instance of the <see cref="NodeImageCollection">NodeImageCollection</see> class containing the elements of the specified source collection.
		''' </summary>
		''' <param name="value">A <see cref="NodeImageCollection">NodeImageCollection</see> with which to initialize the collection.</param>
		Public Sub New(ByVal value As NodeImageCollection)
			MyBase.New()
			AddRange(value)
		End Sub

		''' <summary>
		''' Initializes a new instance of the <see cref="NodeImageCollection">NodeImageCollection</see> class containing the specified array of <see cref="NodeImage">NodeImage</see> objects.
		''' </summary>
		''' <param name="value">An array of <see cref="NodeImage">NodeImage</see> objects with which to initialize the collection. </param>
		Public Sub New(ByVal value As NodeImage())
			MyBase.New()
			AddRange(value)
		End Sub

		''' <summary>
		''' Gets the <see cref="NodeImageCollection">NodeImageCollection</see> at the specified index in the collection.
		''' <para>
		''' In VB.Net, this property is the indexer for the <see cref="NodeImageCollection">NodeImageCollection</see> class.
		''' </para>
		''' </summary>
		Default Public Property Item(ByVal index As Integer) As NodeImage
			Get
				Return CType(List(index), NodeImage)
			End Get
			Set(ByVal Value As NodeImage)
				List(index) = Value
			End Set
		End Property

		''' <summary>
		''' Add an element of the specified <see cref="NodeImage">NodeImage</see> to the end of the collection.
		''' </summary>
		''' <param name="value">An object of type <see cref="NodeImage">NodeImage</see> to add to the collection.</param>
		Public Function Add(ByVal value As NodeImage) As Integer
			Dim index As Integer = List.Add(value)
			If _isTrackingViewState Then
				value.TrackViewState()
				value.SetDirty()
			End If
			Return index
		End Function	   'Add

		''' <summary>
		''' Add an element of the specified <see cref="NodeImage">NodeImage</see> to the end of the collection.
		''' </summary>
		''' <param name="value">An object of type <see cref="NodeImage">NodeImage</see> to add to the collection.</param>
		Public Function Add(ByVal ImageUrl As String) As Integer
			Return Add(New NodeImage(ImageUrl))
		End Function	   'Add

		''' <summary>
		''' Copies the elements of the specified <see cref="NodeImage">NodeImage</see> array to the end of the collection.
		''' </summary>
		''' <param name="value">An array of type <see cref="NodeImage">NodeImage</see> containing the objects to add to the collection.</param>
		Public Sub AddRange(ByVal value As NodeImage())
			For i As Integer = 0 To value.Length - 1
				Add(value(i))
			Next i
		End Sub

		''' <summary>
		''' Adds the contents of another <see cref="NodeImageCollection">NodeImageCollection</see> to the end of the collection.
		''' </summary>
		''' <param name="value">A <see cref="NodeImageCollection">NodeImageCollection</see> containing the objects to add to the collection. </param>
		Public Sub AddRange(ByVal value As NodeImageCollection)
			For i As Integer = 0 To value.Count - 1
				Add(CType(value.List(i), NodeImage))
			Next i
		End Sub

		''' <summary>
		''' Gets the index in the collection of the specified <see cref="NodeImageCollection">NodeImageCollection</see>, if it exists in the collection.
		''' </summary>
		''' <param name="value">The <see cref="NodeImageCollection">NodeImageCollection</see> to locate in the collection.</param>
		''' <returns>The index in the collection of the specified object, if found; otherwise, -1.</returns>
		Public Function IndexOf(ByVal value As NodeImage) As Integer
			Return List.IndexOf(value)
		End Function	   'IndexOf

		''' <summary>
		''' Gets a value indicating whether the collection contains the specified <see cref="NodeImageCollection">NodeImageCollection</see>.
		''' </summary>
		''' <param name="value">The <see cref="NodeImageCollection">NodeImageCollection</see> to search for in the collection.</param>
		''' <returns><b>true</b> if the collection contains the specified object; otherwise, <b>false</b>.</returns>
		Public Function Contains(ByVal value As NodeImage) As Boolean
			' If value is not of type NodeImage, this will return false.
			Return List.Contains(value)
		End Function	   'Contains

		''' <summary>
		''' Add an element of the specified <see cref="NodeImage">NodeImage</see> to the collection at the designated index.
		''' </summary>
		''' <param name="index">An <see cref="system.int32">Integer</see> to indicate the location to add the object to the collection.</param>
		''' <param name="value">An object of type <see cref="NodeImage">NodeImage</see> to add to the collection.</param>
		Public Sub Insert(ByVal index As Integer, ByVal value As NodeImage)
			List.Insert(index, value)
		End Sub	   'Insert

		''' <summary>
		''' Remove the specified object of type <see cref="NodeImage">NodeImage</see> from the collection.
		''' </summary>
		''' <param name="value">An object of type <see cref="NodeImage">NodeImage</see> to remove to the collection.</param>
		Public Sub Remove(ByVal value As NodeImage)
			List.Remove(value)
		End Sub	   'Remove

		''' <summary>
		''' Copies the collection objects to a one-dimensional <see cref="T:System.Array">Array</see> instance beginning at the specified index.
		''' </summary>
		''' <param name="array">The one-dimensional <see cref="T:System.Array">Array</see> that is the destination of the values copied from the collection.</param>
		''' <param name="index">The index of the array at which to begin inserting.</param>
		Public Sub CopyTo(ByVal array As NodeImage(), ByVal index As Integer)
			List.CopyTo(array, index)
		End Sub

		''' <summary>
		''' Creates a one-dimensional <see cref="T:System.Array">Array</see> instance containing the collection items.
		''' </summary>
		''' <returns>Array of type NodeImage</returns>
		Public Function ToArray() As NodeImage()
			Dim arr As NodeImage()
			ReDim Preserve arr(Count - 1)
			CopyTo(arr, 0)

			Return arr
		End Function

#Region "IStateManager Interface"
		''' <summary>
		''' (IStateManager.IsTrackingViewState)
		''' Gets a value indicating whether the NodeImageCollection is tracking its view state changes.
		''' </summary>
		Public ReadOnly Property IsTrackingViewState() As Boolean Implements IStateManager.IsTrackingViewState
			Get
				Return _isTrackingViewState
			End Get
		End Property

		''' <summary>
		''' (IStateManager.TrackViewState)
		''' Instructs the NodeImageCollection to track changes to its view state.
		''' </summary>
		Public Sub TrackViewState() Implements IStateManager.TrackViewState
			_isTrackingViewState = True
		End Sub

		''' <summary>
		''' (IStateManager.SaveViewState)
		''' Saves the changes of NodeImageCollection's view state to an Object.
		''' </summary>
		Public Function SaveViewState() As Object Implements IStateManager.SaveViewState
			If Count = 0 Then
				Return Nothing
			End If
			Dim nodeState(Count) As Object

			Dim index As Integer
			For index = 0 To Count - 1
				nodeState(index) = Me(index).SaveViewState()
			Next index
			Return nodeState
		End Function

		''' <summary>
		''' (IStateManager.LoadViewState)
		''' Loads the NodeImageCollection's previously saved view state.
		''' </summary>
		Public Sub LoadViewState(ByVal state As Object) Implements IStateManager.LoadViewState
			If Not (state Is Nothing) Then
				Dim nodeState As Object() = CType(state, Object())
				Dim index As Integer
				For index = 0 To nodeState.Length - 1
					Dim _nodeImage As New NodeImage
					Add(_nodeImage)

					CType(_nodeImage, IStateManager).TrackViewState()
					CType(_nodeImage, IStateManager).LoadViewState(nodeState(index))
				Next index
			End If
		End Sub
#End Region

	End Class
End Namespace

